Package: MEAL
Title: Perform methylation analysis
Version: 1.43.0
Authors@R: c(person("Carlos", "Ruiz-Arenas", , "carlos.ruiz@isglobal.org", role = c("aut", "cre")),
  person("Juan R.", "Gonzalez", ,"juanr.gonzalez@isglobal.org", role = "aut"))
Maintainer: Xavier Escribà Montagut <xavier.escriba@isglobal.org>
Description: Package to integrate methylation and expression data. It can
    also perform methylation or expression analysis alone. Several plotting
    functionalities are included as well as a new region analysis based on
    redundancy analysis. Effect of SNPs on a region can also be estimated.
Depends: R (>= 3.6.0), Biobase, MultiDataSet
License: Artistic-2.0
biocViews: DNAMethylation, Microarray, Software, WholeGenome
Imports: GenomicRanges, limma, vegan, BiocGenerics, minfi, IRanges,
        S4Vectors, methods, parallel, ggplot2 (>= 2.0.0), permute,
        Gviz, missMethyl, isva, SummarizedExperiment, SmartSVA,
        graphics, stats, utils, matrixStats
Suggests: testthat, IlluminaHumanMethylationEPICanno.ilm10b2.hg19,
        IlluminaHumanMethylation450kanno.ilmn12.hg19, knitr, minfiData,
        BiocStyle, rmarkdown, brgedata
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/MEAL
git_branch: devel
git_last_commit: 438858e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-11
NeedsCompilation: no
Packaged: 2026-05-11 22:44:35 UTC; biocbuild
Author: Carlos Ruiz-Arenas [aut, cre],
  Juan R. Gonzalez [aut]

# Generated by roxygen2: do not edit by hand

export(correlationMethExprs)
export(exportResults)
export(filterResults)
export(getGeneVals)
export(getProbeResults)
export(getRDAresults)
export(plotFeature)
export(plotRDA)
export(plotRegion)
export(runBlockFinder)
export(runBumphunter)
export(runDMRcate)
export(runDiffMeanAnalysis)
export(runDiffVarAnalysis)
export(runPipeline)
export(runRDA)
export(runRegionAnalysis)
export(topRDAhits)
import(Biobase)
import(BiocGenerics)
import(MultiDataSet)
import(methods)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,runValue)
importFrom(S4Vectors,subjectHits)
importFrom(SummarizedExperiment,assay)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,alpha)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(graphics,legend)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,topTable)
importFrom(matrixStats,rowVars)
importFrom(minfi,GenomicRatioSet)
importFrom(minfi,blockFinder)
importFrom(minfi,bumphunter)
importFrom(minfi,cpgCollapse)
importFrom(minfi,getBeta)
importFrom(minfi,ilogit2)
importFrom(minfi,logit2)
importFrom(parallel,mclapply)
importFrom(permute,how)
importFrom(stats,contrasts)
importFrom(stats,cor.test)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,qbeta)
importFrom(stats,qt)
importFrom(utils,write.csv2)
importFrom(vegan,RsquareAdj)
importFrom(vegan,anova.cca)
importFrom(vegan,ordilabel)
importFrom(vegan,rda)

#' Plot results in a genomic region
#' 
#' Plot the results from the different analyses of a \code{ResultSet} in a specific
#' genomic region. It can plot all the results from \code{runPipeline}. 
#' 
#' @details This plot allows to have a quick summary of the methylation or gene
#' expression analyses in a given region. If we use a \code{ResultSet} obtained 
#' from methylation data, transcripts annotation is obtained from archive. If we
#' use a \code{ResultSet} obtained from gene expression data, transcripts annotation
#' is taken from fData. 
#' 
#' This plot can be used to plot the results of one dataset (methylation or gene
#' expression) or to represent the association between methylation and gene 
#' expression data. If only one dataset is used, the p-values and the coefficients 
#' of DiffMean and DiffVar analyses are plotted. If we pass two \code{ResultSet}s, 
#' \code{rset} should contain methylation results and a \code{rset2} the gene expression 
#' results.
#' 
#' @param rset \code{ResultSet}
#' @param range \code{GenomicRanges} with the region coordinates
#' @param results Character with the analyses that will be included in the plot.
#' By default, all analyses available are included. 
#' @param genome String with the genome used to retrieve transcripts annotation:
#' hg19, hg38, mm10. (Default: "hg19")
#' @param tPV Threshold for P-Value
#' @param rset2 Additional \code{ResultSet}
#' @param fNames Names from rset fData
#' @param fNames2 Names from rset2 fData
#' @return Regional plot
#' @export
plotRegion <- function(rset, range, results = names(rset),
                       genome = "hg19", rset2, tPV = 5,
                       fNames = c("chromosome", "start", "end"),
                       fNames2 = c("chromosome", "start", "end")){

  ### Afegir linea per separar blocs. 

  ## Create Basic tracks
  ## Add basic tracks tracks superiores (Idiogram más trancripción)
  basetracks <- list(Gviz::IdeogramTrack(genome = genome, 
                                         chromosome = as.character(seqnames(range))), 
                     Gviz::GenomeAxisTrack())
  
  rset@results <- rset@results[results]
  
  if (missing(rset2)){
    
    
    
    if ("end" %in% colnames(fData(rset)[[1]])){
      endf <- "end"
    } else {
      endf <- "start"
    }
    annot <- GenomicRanges::makeGRangesFromDataFrame(fData(rset)[[1]], 
                                                     seqnames.field = c("chromosome", "seqnames"), 
                                                     start.field = c("position", "start"), 
                                                     end.field = endf)
    if (width(annot[1]) == 1){
      annotTrack <- Gviz::GeneRegionTrack(subsetByOverlaps(annot, range), name = "CpGs", 
                              fill = "black", stacking = "dense",rotation.title = 0)
    } else {
      annotTrack <- Gviz::GeneRegionTrack(subsetByOverlaps(annot, range), name = "Transcripts", 
                                          symbol = names(subsetByOverlaps(annot, range)), 
                                          fill = "lightblue", 
                                          gene = names(subsetByOverlaps(annot, range)),
                                          showId = TRUE, geneSymbol = TRUE,
                                          shape = "arrow", transcriptAnnotation = "symbol",
                                          collapseTranscripts = TRUE) 
    }
  } else {
    rset2@results <- rset2@results[names(rset2) %in% results]
    
    annot <- GenomicRanges::makeGRangesFromDataFrame(fData(rset)[[1]], 
                                                     seqnames.field = c("chromosome", "seqnames"), 
                                                     start.field = c("position", "start"), 
                                                     end.field = "start")
    annot2 <- GenomicRanges::makeGRangesFromDataFrame(fData(rset2)[[1]], 
                                                      seqnames.field = c("chromosome", "seqnames"), 
                                                      start.field = c("position", "start"), 
                                                      end.field = "end")
    annotTrack <- c(
      Gviz::GeneRegionTrack(subsetByOverlaps(annot2, range), name = "Transcripts", 
                            symbol = names(subsetByOverlaps(annot2, range)), 
                            fill = "lightblue", 
                            gene = names(subsetByOverlaps(annot2, range)),
                            showId = TRUE, geneSymbol = TRUE,
                            shape = "arrow", transcriptAnnotation = "symbol",
                            collapseTranscripts = TRUE), 
      Gviz::GeneRegionTrack(subsetByOverlaps(annot, range), name = "CpGs", 
                            fill = "black", stacking = "dense",rotation.title = 0)) 
  }
  resultTrack <- list()
  
  if (missing(rset2)){
    
    if ("DiffMean" %in% names(rset)){
      diffMean <- getProbeResults(rset, rid = "DiffMean", fNames = c("chromosome", "start", "end"))
      diffMeanGR <- GenomicRanges::makeGRangesFromDataFrame(
        diffMean, seqnames.field = c("chromosome", "seqnames"), 
        start.field = "start", 
        end.field = "end", keep.extra.columns = TRUE)
      diffMeanGR <- subsetByOverlaps(diffMeanGR, range)
      resultTrack <- c(resultTrack, Gviz::DataTrack(range = diffMeanGR, 
                                                    data = diffMeanGR$logFC, 
                                                    genome = genome, name = "DiffMean", type = "h", 
                                                    baseline = 0),
                       Gviz::DataTrack(range = diffMeanGR, data = -log10(diffMeanGR$P.Value), 
                                       genome = genome, name = "DiffMean -log10 p-value", 
                                       type = "p", col = "darkblue",
                                       baseline = tPV))
    }
    
    
    if ("DiffVar" %in% names(rset)){
      DiffVar <- getProbeResults(rset, rid = "DiffVar", 
                                 fNames = c("chromosome", "start", "end"))
      DiffVarGR <- GenomicRanges::makeGRangesFromDataFrame(
        DiffVar, seqnames.field = c("chromosome", "seqnames"), 
        start.field = "start", 
        end.field = "end", keep.extra.columns = TRUE)
      DiffVarGR <- subsetByOverlaps(DiffVarGR, range)
      resultTrack <- c(resultTrack, Gviz::DataTrack(
        range = DiffVarGR, data = DiffVarGR$DiffLevene, 
        genome = genome, name = "DiffVar", type = "h", baseline = 0, col = "darkgreen"),
        Gviz::DataTrack(range = DiffVarGR, data = -log10(DiffVarGR$P.Value), 
                        baseline = tPV, genome = genome, name = "DiffVar -log10 p-value", 
                        type = "p", col = "darkolivegreen"))
    }
  } else {
    if ("DiffMean" %in% names(rset)){
      diffMean <- getProbeResults(rset, rid = "DiffMean", fNames = c("chromosome", "start", "end"))
      diffMeanGR <- GenomicRanges::makeGRangesFromDataFrame(
        diffMean, seqnames.field = c("chromosome", "seqnames"), 
        start.field = "start", 
        end.field = "end", keep.extra.columns = TRUE)
      diffMeanGR <- subsetByOverlaps(diffMeanGR, range)
      resultTrack <- c(resultTrack, Gviz::DataTrack(range = diffMeanGR, 
                                                    data = diffMeanGR$logFC, 
                                                    genome = genome, 
                                                    name = "DiffMean Res1", type = "h", 
                                                    baseline = 0, 
                                                    col = "black"))
    }
    if ("DiffMean" %in% names(rset2)){
      diffMean <- getProbeResults(rset2, rid = "DiffMean", fNames = fNames2)
      diffMeanGR <- GenomicRanges::makeGRangesFromDataFrame(
        diffMean, seqnames.field = c("chromosome", "seqnames"), 
        start.field = fNames2[2], 
        end.field = fNames2[3], keep.extra.columns = TRUE)
      diffMeanGR <- subsetByOverlaps(diffMeanGR, range)
      resultTrack <- c(resultTrack, Gviz::DataTrack(range = diffMeanGR, 
                                                    data = diffMeanGR$logFC, 
                                                    genome = genome, name = "DiffMean Res2", type = "h", 
                                                    baseline = 0))
    }
    
    if ("DiffVar" %in% names(rset)){
      DiffVar <- getProbeResults(rset, rid = "DiffVar", fNames = fNames)
      DiffVarGR <- GenomicRanges::makeGRangesFromDataFrame(
        DiffVar, seqnames.field = c("chromosome", "seqnames"), 
        start.field = fNames[2], 
        end.field = fNames[3], keep.extra.columns = TRUE)
      DiffVarGR <- subsetByOverlaps(DiffVarGR, range)
      resultTrack <- c(resultTrack, Gviz::DataTrack(
        range = DiffVarGR, data = DiffVarGR$DiffLevene, col = "black",
        genome = genome, name = "DiffVar Res1", type = "h", baseline = 0))
    }
    if ("DiffVar" %in% names(rset2)){
      DiffVar <- getProbeResults(rset2, rid = "DiffVar", fNames = fNames)
      DiffVarGR <- GenomicRanges::makeGRangesFromDataFrame(
        DiffVar, seqnames.field = c("chromosome", "seqnames"), 
        start.field = fNames[2], 
        end.field = fNames[3], keep.extra.columns = TRUE)
      DiffVarGR <- subsetByOverlaps(DiffVarGR, range)
      resultTrack <- c(resultTrack, Gviz::DataTrack(
        range = DiffVarGR, data = DiffVarGR$DiffLevene, 
        genome = genome, name = "DiffVar Res2", type = "h", baseline = 0))
    }
  }
  
  dmrTracks <- list()
  # Create tracks from Region Analysis
  if ("bumphunter" %in% names(rset)){
    bumphunter <- MultiDataSet::getAssociation(rset, rid = "bumphunter")
    if (nrow(bumphunter) != 0){
      bumphunterGR <- GenomicRanges::makeGRangesFromDataFrame(
        bumphunter, seqnames.field = "chr", start.field = "start",
        end.field = "end", keep.extra.columns = TRUE)
      bumphunterGR$id <- names(bumphunterGR)
      if (!"p.value" %in% colnames(bumphunter)){
        bumphunterGR$col <- "null"
      } else {
        bumphunterGR$col <- ifelse(bumphunter$p.value < 0.05, "sig", "no.sig")
      }
      bumphunterGR <- subsetByOverlaps(bumphunterGR, range)
      bumptrack <- Gviz::AnnotationTrack(bumphunterGR,
                                         name = "Bumphunter", shape = "box",
                                         group = bumphunterGR$id, 
                                         feature = bumphunterGR$col,
                                         rotation.title = 0, 
                                         cex.title = 0.6)
      dmrTracks <- c(dmrTracks, bumptrack)
    }
  }
  
  if ("blockFinder" %in% names(rset)){
    blockFinder <- MultiDataSet::getAssociation(rset, rid = "blockFinder")
    if (nrow(blockFinder) != 0){
      blockFinderGR <- GenomicRanges::makeGRangesFromDataFrame(
        blockFinder, seqnames.field = "chr", start.field = "start", end.field = "end",
        keep.extra.columns = FALSE)
      blockFinderGR$id <- names(blockFinderGR)
      if (!"p.value" %in% colnames(blockFinder)){
        blockFinderGR$col <- "null"
      } else {
        blockFinderGR$col <- ifelse(blockFinder$p.value < 0.05, "sig", "no.sig")
      }
      blockFinderGR <- subsetByOverlaps(blockFinderGR, range)
      blockTrack <- Gviz::AnnotationTrack(blockFinderGR,
                                          name = "blockFinder", shape = "box",
                                          group = blockFinderGR$id, 
                                          feature = blockFinderGR$col,
                                          rotation.title = 0, 
                                          cex.title = 0.6)
      dmrTracks <- c(dmrTracks, blockTrack)
    }
  }
  
  if ("dmrcate" %in% names(rset)){
    dmrcate <- MultiDataSet::getAssociation(rset, rid = "dmrcate")
    if (nrow(dmrcate) != 0){
      colnames(dmrcate)[colnames(dmrcate) == "Stouffer"] <- "p.val"
      dmrcateGR <- GRanges(dmrcate$coord)
      mcols(dmrcateGR) <- dmrcate[, -1]
      dmrcateGR$id <- 1:nrow(dmrcate)
      dmrcateGR$col <- ifelse(dmrcateGR$p.val < 0.05, "sig", "no.sig")
      dmrcateGR <- subsetByOverlaps(dmrcateGR, range)
      dmrcateTrack <- Gviz::AnnotationTrack(dmrcateGR,
                                            name = "DMRcate", shape = "box", genome = genome,
                                            group = dmrcateGR$id,  
                                            cex.title = 0.6, 
                                            feature = dmrcateGR$col,
                                            rotation.title = 0, showTitle = TRUE)
      dmrTracks <- c(dmrTracks, dmrcateTrack)
    }
  }
  Gviz::plotTracks(c(basetracks, annotTrack, dmrTracks, resultTrack), 
                   from = start(range), to = end(range), title.width = 1.5,
                   groupAnnotation = "group", just.group = "above", 
                   null = "white", sig = "forestgreen", no.sig = "grey")
}

# context("Plotting")
# 
# library(minfiData)
# set <- prepareMethylationSet(getBeta(MsetEx)[1:10,], pheno = data.frame(pData(MsetEx)))
# methyOneVar <- DAPipeline(set, variable_names = "sex", probe_method = "ls")
# methyTwoVar <- DAPipeline(set, variable_names =  "person", probe_method = "ls")
# 
# range <- GenomicRanges::GRanges(seqnames=Rle("chrY"), 
#                                 ranges = IRanges(3000000, end=12300000))
# rangeNoSNPs <- DARegionAnalysis(set, variable_names = "sex", range = range)
# 
# eset <- ExpressionSet(matrix(runif(12, max = 15), 3))
# annot <- data.frame(chromosome = rep("chrY", 3), start = c(10000, 4500000, 5000000), 
#                     end = c(100090, 6000000, 90000000))
# fData(eset) <- annot
# pData(eset) <- data.frame(sex = c("H", "M", "H", "M"))
# esetRes <- DAPipeline(eset, variable_names = "sex", probe_method = "ls")
# 
# test_that("Plot CPGs",{
#   expect_error(plotFeature(methyOneVar, 12351413654), "feat index must be greater than 0 and smaller than the number of features.")
#   expect_error(plotFeature(methyOneVar, -12), "feat index must be greater than 0 and smaller than the number of features.")
#   expect_error(plotFeature(methyOneVar, 1, variables = character()), "variables must have one or two values.")
#   expect_error(plotFeature(methyOneVar, 1, variables = "character"),"Not all variables are present in set phenodata.")
#   expect_error(plotFeature(methyOneVar, 1, variables = c("age", "sex", "status")), "variables must have one or two values.")
#   expect_error(plotFeature(methyOneVar, "12"), "feat name is not present in the set.")
#   expect_error(plotFeature(methyOneVar, numeric()), "feat must contain only one value.")
#   expect_error(plotFeature(methyOneVar, 1:5), "feat must contain only one value.")
#   expect_error(plotFeature(MsetEx, 12, "age"), "set must be of class AnalysisResults, MethylationSet or ExpressionSet")
#   expect_error(plotBestFeatures(MsetEx, 12, "age"), "set must be of class AnalysisResults, ExpressionSet or MethylationSet")
#   expect_warning(plotBestFeatures(rangeNoSNPs, 12421531461, "sex"), "n is greater than the number of betas present in the set.")
# })
# 
# test_that("Plot Ewas", {
#   expect_error(plotEWAS(methyOneVar, variable = character()), "variable must have one value.")
#   expect_error(plotEWAS(methyOneVar, variable = c("a", "b")), "variable must have one value.")
#   expect_error(plotEWAS(methyOneVar, variable = "a"), "Variable is not present in the model.")
#   expect_warning(plotEWAS(rangeNoSNPs), "For a better visualization of the region, use plotRegion.")
#   plotEWAS(methyOneVar)
# })
# 
# test_that("Plot Volcano", {
#   plotVolcano(methyOneVar)
#   expect_error(plotVolcano(methyOneVar, variable = character()), "variable must have one value.")
#   expect_error(plotVolcano(methyOneVar, variable = c("a", "b")), "variable must have one value.")
#   expect_error(plotVolcano(methyOneVar, variable = "a"), "Variable is not present in the model.")
# })
# 
# test_that("QQPlot", {
#   expect_error(plotQQ(methyOneVar, variable = character()), "variable must have one value.")
#   expect_error(plotQQ(methyOneVar, variable = c("a", "b")), "variable must have one value.")
#   expect_error(plotQQ(methyOneVar, variable = "a"), "Variable is not present in phenodata.")
#   expect_warning(plotQQ(rangeNoSNPs), "QQplot is not recommended in range analyis.")
# })
# 
# test_that("plotRDA", {
#   expect_error(plotRDA(rangeNoSNPs, n_feat = -1), "n_feat must be greater than 1.")
#   expect_warning(plotRDA(rangeNoSNPs, n_feat = 9999), "n_feat is greater than the total number of features in the range.")
# })
# 
# test_that("Plot Region", {
#   expect_error(plotRegion(rangeNoSNPs, variable = character()), "variable must have one value.")
#   expect_error(plotRegion(rangeNoSNPs, variable = c("a", "b")), "variable must have one value.")
#   expect_error(plotRegion(rangeNoSNPs, variable = "a"), "Variable is not present in modelVariables.\nValid variables are sex")
#   expect_error(plotRegion(methyOneVar), "range must be present to use plotRegion with a AnalysisResults")
#   expect_error(plotRegion(esetRes), "Results must have a column called position to perform this plot.")
#   plotRegion(rangeNoSNPs)
# })
# 
# dev.off()
# file.remove("Rplots.pdf")

#' Run differential mean analysis
#' 
#' Run differential mean analysis using t-moderated statistics. This function relies
#' on \code{lmFit} from limma package.
#' 
#' @export runDiffMeanAnalysis
#' 
#' @param set Matrix, \code{GenomicRatioSet}, \code{SummarizedExperiment} or 
#' \code{ExpressionSet}. 
#' @param model Model matrix or formula to get model matrix from \code{set}. 
#' @param weights weights used in the lmFit model.
#' @param method String indicating the method used in the regression: "ls" or 
#' "robust". (Default: "ls")
#' @param max_iterations Numeric indicating the maximum number of iterations
#' done in the robust method. 
#' @param betas If \code{set} is a \code{GenomicRatioSet}, should beta values be
#' used? (Default: TRUE)
#' @param resultSet Should results be encapsulated in a \code{resultSet}? (Default: TRUE)
#' @param warnings Should warnings be displayed? (Default:TRUE) 
#' @return \code{MArrayLM} or \code{resultSet} with the result of the differential
#' mean analysis.
#' @examples
#' if (require(minfiData)){
#'  mvalues <- getM(MsetEx)[1:100, ]
#'  model <- model.matrix(~ Sample_Group, data = pData(MsetEx)) 
#'  res <- runDiffMeanAnalysis(mvalues, model, method = "ls")
#'  res
#' }
runDiffMeanAnalysis <- function(set, model, weights = NULL,  method = "ls", max_iterations = 100, 
                    betas = TRUE, resultSet = TRUE, warnings = TRUE) {
  
  ## Create model matrix from formula
  if (is(model, "formula")){
    model <- createModel(set, model)
    set <- set[, rownames(model)]
  }
  
 method <- match.arg(method, choices = c("ls", "robust"))

  ## Get matrix
  if (is(set, "ExpressionSet")){
    mat <- Biobase::exprs(set)
  } else if (is(set, "GenomicRatioSet")){
    mat <- minfi::getBeta(set)
    if (!betas) {
      mat[mat == 0] <- 1e-3
      mat[mat == 1] <- 1 - 1e-3
      mat <- minfi::logit2(mat)
    }
  } else if (is(set, "SummarizedExperiment")){
    mat <- SummarizedExperiment::assay(set)
  } else if (is.matrix(set)){
    mat <- set
  } else {
    stop("set must be a matrix, ExpressionSet, MethylationSet, GenomicRatioSet or SummarizedExperiment.")
  }
  
  fFun <- getFeatureDataFun(set)

  if (length(max_iterations) == 0 || !is(max_iterations, "numeric") || max_iterations < 1){
    warning("max_iterations must be a numeric greater than 1. The default value (100) will be used.")
    max_iterations <- 100
  }

  fit <- limma::lmFit(mat, model, method = method, maxit = max_iterations, weights = weights)

  if (resultSet){
    fit <- MultiDataSet::create_resultset(fOrigin = "DAProbe", fData = list(main = fFun(set)), 
                                   lResults = list(DiffMean = list(result = fit)))
  } 
  fit
}

context("Probe Analysis")

library(minfiData)
library(minfi)
miniset <- MsetEx[1:10, ]
set <- ratioConvert(mapToGenome(miniset))

test_that("DiffMean", {
  probes <- runDiffMeanAnalysis(set = set, model = ~ status)
  expect_match(class(probes), "ResultSet")
  
  probes <- runDiffMeanAnalysis(set = set, model = ~ status, resultSet = FALSE)
  expect_match(class(probes), "MArrayLM")
  expect_equal(as.integer(nrow(set)), nrow(probes))
  
  beta <- getBeta(set)
  model <- model.matrix(~ status, pData(set))
  probes <- runDiffMeanAnalysis(set = beta, model = model)
  expect_match(class(probes), "ResultSet")
})

test_that("DiffVar", {
  probes <- runDiffVarAnalysis(set = set, model = ~ status)
  expect_match(class(probes), "ResultSet")
  
  probes <- runDiffVarAnalysis(set = set, model = ~ status, resultSet = FALSE)
  expect_match(class(probes), "MArrayLM")
  expect_equal(as.integer(nrow(set)), nrow(probes))
  
  beta <- getBeta(set)
  model <- model.matrix(~ status, pData(set))
  probes <- runDiffVarAnalysis(set = beta, model = model)
  expect_match(class(probes), "ResultSet")
})


eset <- ExpressionSet(matrix(rnorm(1000), ncol = 10))
pData(eset) <- data.frame(sex = rep(c("H", "M"), each = 5))


test_that("Probe Analysis works with ExpressionSet", {
  probes <- runDiffMeanAnalysis(set = eset, model = ~sex)
  expect_match(class(probes), "ResultSet")
  
  probes <- runDiffMeanAnalysis(set = eset, model = ~ sex, resultSet = FALSE)
  expect_match(class(probes), "MArrayLM")
  expect_equal(as.integer(nrow(eset)), nrow(probes))
  
  probes <- runDiffVarAnalysis(set = eset, model = ~sex)
  expect_match(class(probes), "ResultSet")
  
  probes <- runDiffVarAnalysis(set = eset, model = ~ sex, resultSet = FALSE)
  expect_match(class(probes), "MArrayLM")
  expect_equal(as.integer(nrow(eset)), nrow(probes))
})


# 
# test_that("Empty variables", {
#   emptyset <- new(Class = "MethylationSet")
#   emptyeset <- new(Class = "ExpressionSet")
#   emptymodel <-  matrix(ncol = 0, nrow = 0)
#   expect_error(DiffMeanAnalysis(set = emptyset, model = emptymodel), "The set is empty.")
#   expect_error(DiffMeanAnalysis(set = set, model = emptymodel), "The model matrix is empty.")
#   expect_error(DiffMeanAnalysis(set = emptyset, model = model), "The set is empty.")
#   expect_error(DiffMeanAnalysis(set = emptyeset, model = model), "The set is empty.")
# })
# 
# test_that("Wrong variables", {
#   expect_error(DiffMeanAnalysis(set = set, model= model[1:2, ]), 
#                "The number of samples is different in the set and in the model")
#   expect_error(DiffMeanAnalysis(set = set, model= model, method = character()), 
#                "method should be one of \"ls\", \"robust\"")
#   expect_error(DiffMeanAnalysis(set = set, model= model, method = "character"), 
#                "method should be one of \"ls\", \"robust\"")
#   expect_error(DiffMeanAnalysis(set = set, model= model, method = c("ls", "robust")), 
#                "method should be one of \"ls\", \"robust\"")
#   expect_warning(DiffMeanAnalysis(set = set, model= model, max_iterations = -1), 
#                "max_iterations must be a numeric greater than 1. The default value will be used.")
#   expect_warning(DiffMeanAnalysis(set = set, model= model, max_iterations = "1"), 
#                  "max_iterations must be a numeric greater than 1. The default value will be used.")
#   expect_warning(DiffMeanAnalysis(set = set, model= model, max_iterations = numeric()), 
#                  "max_iterations must be a numeric greater than 1. The default value will be used.")
# })

#' Filter the data.frame obtained from probe analysis
#' 
#' @param results Data.frame with the results of probe analysis
#' @param range \code{GenomicRanges} with the desired range. 
#' @param position Character with the name of the column containing the positions
#' @param chr Character with the name of the column containing the chromosome
#' @return Data.frame with the results of the probes of the range
#' @export
filterResults <- function(results, range, position = "position", chr = "chromosome"){
  if(ncol(results) == 0 | nrow(results) == 0){
    stop("results is empty")
  }
    
  if (!is(range, "GenomicRanges")){
    stop("range must be a GenomicRanges object.")
  }
  if (length(range) == 0){
    stop("range is empty.")
  }
  
  if (length(range) != 1){
    stop("range must be a GenomicRanges with only one range.")
  }
      
  maskResults <- as.vector(results[ , chr] == as.character(S4Vectors::runValue(GenomicRanges::seqnames(range)))) & 
    results[ , position] >= start(range) &  results[ , position] <= end(range)
  results <- results[maskResults, ]
  if (nrow(results) == 0){
    warning("There are no cpgs in the range. An empty data.frame will be returned.")
  }
  return(results)
}

# context("Filtering")
# 
# 
# library(minfiData)
# 
# range <- GenomicRanges::GRanges(seqnames=Rle("chrY"), 
#                                 ranges = IRanges(3000000, end = 12300000))
# set <- prepareMethylationSet(MsetEx[1:100, ], data.frame(pData(MsetEx)))
# 
# eset <- ExpressionSet(matrix(runif(6, max = 15), 4))
# annot <- data.frame(chromosome = c(rep("chrY", 3), "chr1" ), start = c(10000, 4500000, 5000000, 10000), 
#                     end = c(100090, 6000000, 90000000, 100090))
# fData(eset) <- annot
# 
# 
# test_that("Results filtering", {
#     methyOneVar <- DAPipeline(set, variable_names = "sex", probe_method = "ls")
#     
#   filteredResults <- filterResults(probeResults(methyOneVar), range)
#   expect_match(class(filteredResults), "data.frame")
#   expect_equal(nrow(filteredResults), 32)
#   filteredResults <- filterResults(probeResults(methyOneVar), range)
#   expect_match(class(filteredResults), "data.frame")
#   expect_equal(nrow(filteredResults), 32)
# })
# 
# test_that("Results filtering, empty variables", {
#     methyOneVar <- DAPipeline(set, variable_names = "sex", probe_method = "ls")
#     
#   emptyRes <- data.frame()
#   emptyrange <- GRanges()
#   expect_error(filterResults(emptyRes, emptyrange), "results is empty")
#   expect_error(filterResults(probeResults(methyOneVar), emptyrange), "range is empty")
# })
# 
# test_that("Results filtering, wrong variables", {
#     methyOneVar <- DAPipeline(set, variable_names = "sex", probe_method = "ls")
#     
#   outrange <- GenomicRanges::GRanges(seqnames=Rle("chrM"), 
#                                      ranges = IRanges(30000, end=123000000))
#   expect_warning(filterResults(probeResults(methyOneVar), outrange), "There are no cpgs in the range. An empty data.frame will be returned.")
#   outrange <- data.frame(chr = "chr1", start = 19051, end = 1251905)
#   expect_error(filterResults(probeResults(methyOneVar), outrange), "range must be a GenomicRanges object.")
#   outrange <- GenomicRanges::GRanges(seqnames=Rle(c("chr1", "chr2")), 
#                                      ranges = IRanges(c(30000, 2149421), end=c(123000000, 234841418)))
#   expect_error(filterResults(probeResults(methyOneVar), outrange), "range must be a GenomicRanges with only one range.")
# }

#' Run different DMR detection methods
#' 
#' @export runRegionAnalysis
#' @details This function has been deprecated and will be defunct in the new version.
#'  
#' 
#' @param set \code{GenomicRatioSet}, \code{eSet} derived object or 
#' \code{SummarizedExperiment}
#' @param model Model matrix representing a linear model.
#' @param methods Character vector with the names of the methods used to estimate 
#' the regions. Valid names are: "blockFinder", "bumphunter" and "DMRcate".
#' @param coefficient Numeric with the index of the model matrix used to perform
#' the analysis.
#' @param bumphunter_params List with other parameter passed to \code{runBumphunter} 
#' function.
#' @param blockFinder_params List with other parameter passed to \code{runBlockFinder} 
#' function.
#' @param dmrcate_params List with other parameter passed to \code{runDMRcate} 
#' function.
#' @param verbose Logical value. Should the function be verbose? (Default: FALSE)
#' @param resultSet Should results be encapsulated in a \code{resultSet}? (Default: TRUE)
#' @return List or \code{resultSet} with the result of the DMR detection methods.
#' @seealso \code{\link[minfi]{bumphunter}}, \code{\link[minfi]{blockFinder}}, 
#' \code{\link[DMRcate]{dmrcate}} 
#' @examples
#' if (require(minfiData)){
#' set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#'  model <- model.matrix(~Sample_Group, data = pData(MsetEx))
#'  res <- runRegionAnalysis(set, model) 
#'  res
#' }
runRegionAnalysis <- function (set, model, methods = c("blockFinder", "bumphunter", "DMRcate"), 
                            coefficient = 2, bumphunter_params = NULL, 
                            blockFinder_params = NULL, dmrcate_params = NULL,
                            verbose = FALSE, resultSet = TRUE)
{
  .Deprecated()
  
}

#' Compute signification of RDA test
#' 
#' Compare R2 obtained in our region of interest with the global R^2 and the R^2 of regions with 
#' the same number of probes.
#' @param fullMat Matrix with the whole genome expression or methylation values
#' @param varsmodel Matrix with the model
#' @param covarsmodel Matrix with the covariables model
#' @param featNum Numeric with the number of features of the RDA model
#' @param R2 Numeric with the R2 of the RDA model
#' @param num_permutations Numeric with the number of permutations.
#' @return Numeric vector with the probability of finding a region with the same number of probes
#' with a bigger R2 and the global R2.
computeRDAR2 <- function(fullMat, varsmodel, covarsmodel = NULL, featNum, R2, 
                         num_permutations = 1e5-1){

  allvar <- matrixStats::rowVars(fullMat)

   ## If there are covariables, substract their effect prior to adjusting 
  if (!is.null(covarsmodel)){
    covfit <- limma::fitted.MArrayLM(limma::lmFit(fullMat, covarsmodel))
    fullMat <- fullMat - covfit
  }
  fit <- limma::fitted.MArrayLM(limma::lmFit(fullMat, varsmodel))
  varfit <- matrixStats::rowVars(fit)
  inds <- sample(1:(nrow(fullMat) - featNum), num_permutations, replace = TRUE)
  
  ## Compute the R2 explained by groups with the same number of features than the region of interest
  randomR2 <- vapply(inds, function(x) { 
      R2 <- sum(varfit[x:(x + featNum - 1)])/sum(allvar[x:(x + featNum - 1)])
      }, numeric(1))
  randomR2 <- c(randomR2, R2)
  pval <- sum(randomR2 >= R2)/(num_permutations + 1)

  globalR2 <- sum(varfit)/sum(allvar)
  c(globalpval = pval, globalR2 = globalR2)
}

#' Run DMRcate
#' 
#' @details This function has been deprecated and will be defunct in the new version.
#' @export
#' 
#' @param set \code{GenomicRatioSet}, \code{eSet} derived object or 
#' \code{SummarizedExperiment}
#' @param model Model matrix or formula to get model matrix from \code{set}. 
#' @param coefficient Numeric with the column of model matrix used in the analysis.
#' (Default: 2)
#' @param resultSet Should results be encapsulated in a \code{resultSet}? (Default: TRUE)
#' @param ... Further arguments passed to \code{cpg.annotate} or \code{dmrcate}.
#' @return data.frame or \code{resultSet} with the result of \code{bumphunter}
#' @seealso \code{\link[DMRcate]{dmrcate}}, \code{\link[DMRcate]{cpg.annotate}}
runDMRcate <- function(set, model, coefficient = 2, resultSet = FALSE, ...){
  
  .Deprecated()
}

#' Plot RDA results
#' 
#' @name plotRDA
#' @aliases plotRDA
#' @export
#' 
#' @param object \code{ResultSet}
#' @param pheno data.frame with the variables used to color the samples. 
#' @param n_feat Numeric with the number of cpgs to be highlighted. Default: 5.
#' @param main Character with the plot title.
#' @param alpha Numeric with the alpha level for colour transparance. Default: 1; no
#' transparency.
#' @return A plot is generated on the current graphics device.
#' @examples
#' if (require(minfiData)){
#' set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#' model <- model.matrix(~set$sex)
#' rda <- runRDA(set, model)
#' plotRDA(rda, pheno = data.frame(factor(set$sex)))
#' }
plotRDA <- function(object, pheno = data.frame(), n_feat = 5, main = "RDA plot", 
                    alpha = 1){

  stopifnot("RDA" %in% names(object))
  ans <- getAssociation(object, rid = "RDA")
  
  if (is.null(dim(pheno))){
    pheno <- data.frame(pheno)
  }
  
  classes <- vapply(pheno, class, character(1))
  pheno[, classes == "character"] <- lapply(pheno[, classes == "character", drop = FALSE], function(x) factor(x))
  
  classes <- vapply(pheno, class, character(1))
  factormatrix <- pheno[, classes == "factor", drop = FALSE]
  phenocont <- pheno[, classes %in% c("numeric", "integer"), drop = FALSE]
  factor <- as.vector(sapply(colnames(factormatrix), function(x) levels(factormatrix[, x])))
  
  
  
  if (ncol(factormatrix) > 1){
    factormatrix[] <- lapply(factormatrix, as.character)
    
    phenofactor <- as.factor(sapply(rownames(factormatrix),
                               function(x) paste0(unlist(factormatrix[x, ]), collapse = "")))
  } else if (ncol(factormatrix) == 1) {
    phenofactor <- factormatrix[, 1]
  } else {
    phenofactor <- 1
  }
  
  r2 <- ans$rdaR2
  pval <- ans$pval
  if (pval < 1e-3){
    pval <- format(signif(pval, 3), scientific = TRUE)
  } else{
    pval <- round(pval, 3)
  }
  
  ans$CCA$centroids <- NULL
  if (length(factor)){
    ans$CCA$centroids <- t(data.matrix(data.frame(
      lapply(colnames(factormatrix), function(x) 
        getCentroids(ans, factormatrix[ , x])),
      check.names = FALSE)))
  }
  
  ## Remove from biplot variables already present in centroids
  ans$CCA$biplot[] <- 0
  
  if (ncol(phenocont)) {
    xx <- data.matrix(phenocont)
    ans$CCA$biplot <- (1/sqrt(colSums(xx^2))) * crossprod(xx, ans$CCA$u)
  }

  temp <- ans$CCA$v
  
  if (ncol(temp) == 1){
    temp <- cbind(temp, ans$CA$v[ , 1])
  }
  o1 <- order(abs(temp[,1]), decreasing=TRUE)
  o2 <- order(abs(temp[,2]), decreasing=TRUE)
  
  plot(ans, display=c("sp", "cn", "wa"), type="n", main = main, scaling = 3)
  
  filter <- union(o1[1:n_feat], o2[1:n_feat])
    
  text(ans, display = "species", select = filter, cex=0.6, scaling = 3)
  
  ## Set points
  pch = (15:18)[as.numeric(phenofactor)]
  points(ans, display = "wa", col = ggplot2::alpha(as.numeric(phenofactor), alpha), 
         pch = pch, scaling = 3)
  
  if (length(factor)){
    vegan::ordilabel(ans, display = "cn", col = "blue", label = factor)
  }
  
  if (ncol(phenocont)){
    text(ans, display = "bp", col = "blue", scaling = 3)
  }
  
  legend("topleft", c(as.expression(bquote(R^2 ~ "=" ~ .(round(r2, 3)))), paste("p-value:", pval), levels(phenofactor)), 
         col = c("white", "white", ggplot2::alpha(1:nlevels(phenofactor), alpha)), 
         cex = 0.8, bty = "n", pch = c(1, 1, c(15:18)[seq_len(nlevels(phenofactor))]))
}


getCentroids <- function(rda, factor){
  posRDA <- data.frame(rda$CCA$wa)
  splitpos <- split(posRDA, factor)
  if (is.null(dim(splitpos[[1]]))){
    splitpos <- lapply(splitpos, function(RDA1) data.frame(RDA1))
  }
  pos <- data.frame(lapply(splitpos, colMeans))
  colnames(pos) <- names(splitpos)
  colnames(pos) <- paste0(colnames(factor), colnames(pos))
  return(pos)
}

#' Get a summary of RDA results
#' 
#' Get statistics from RDA result.
#'  
#'  
#' @aliases getRDAresults
#' @export
#' @name getRDAresults
#' @param object \code{ResultSet}
#' @return Numeric vector with the RDA statistics
getRDAresults <- function(object) {
    
  stopifnot(is(object, "ResultSet"), "RDA" %in% names(object))
    rda <- getAssociation(object, rid = "RDA")
    res <- c(R2 = rda$rdaR2, pval = rda$pval, global.R2 = unname(rda$globalR2), 
                global.pval = unname(rda$globalpval))
    
    return(res)
  }

#' Perform differential methylation analysis
#' 
#' Wrapper for analysing differential methylation and expression at region and probe level.
#' 
#' @details This function is the main wrapper of the package. First, it simplifies the 
#' the set to only contain the common samples between phenotype and features. In addition,
#' it allows to change the class of the variables and to apply genomic models (more 
#' information on \code{preparePhenotype}). Afterwards, analysis per probe and per 
#' region are done merging the results in an \code{AnalysisResults} object.
#' 
#' Default linear model will contain a sum of the variables and covariables. If 
#' interactions are desired, a costum formula can be specified. In that case, variables
#' and covariables must also be specified in order to assure the proper work of the
#' resulting \code{AnalysisResult}. In addition, the number of variables of the model
#' for which the calculation will be done \strong{must} be specified. 
#' 
#' @export runPipeline
#' @param set \code{GenomicRatioSet}, \code{eSet} derived object or 
#' \code{SummarizedExperiment}
#' @param variable_names Character vector with the names of the variables that
#' will be returned as result.
#' @param covariable_names Character vector with the names of the variables that
#' will be used to adjust the model. 
#' @param model Model matrix or formula to get model matrix from \code{set}. 
#' @param weights weights used in the lmFit model (default NULL)
#' @param num_vars Numeric with the number of variables in the matrix for which the
#' analysis will be performed. Compulsory if equation is not null. 
#' @param sva Logical. Should Surrogate Variable Analysis be applied? 
#' (Default: FALSE) 
#' @param betas If \code{set} is a \code{GenomicRatioSet}, should beta values be
#' used? (Default: TRUE)
#' @param range \code{GenomicRanges} with the region used for RDA
#' @param analyses Vector with the names of the analysis to be run (DiffMean and/or DiffVar).
#' @param DiffMean_params List with other parameter passed to \code{runBumphunter} 
#' function.
#' @param DiffVar_params List with other parameter passed to \code{runBumphunter} 
#' function.
#' @param rda_params List with other parameter passed to \code{runRDA} 
#' function.
#' @param method String indicating the method used in the regression: "ls" or 
#' "robust". (Default: "ls")
#' @param big Logical value indicating whether SmartSVA should be instead of SVA 
#' (TRUE recommended for methylation or when having large number of samples). 
#' Default is FALSE. 
#' @param verbose Logical value. If TRUE, it writes out some messages indicating progress. 
#' If FALSE nothing should be printed.
#' @param warnings Should warnings be displayed? (Default:TRUE) 
#' @return \code{ResultSet} object 
#' @examples
#' if (require(minfiData)){
#' set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#'  res <- runPipeline(set, variable_names = "Sample_Group") 
#'  res
#' }
runPipeline <-  function(set, variable_names, 
                         covariable_names = NULL, 
                         model = NULL, weights = NULL, num_vars,  
                         sva = FALSE, betas = TRUE, range,
                         analyses = c("DiffMean"),
                         verbose = FALSE, warnings = TRUE, 
                         DiffMean_params = NULL, DiffVar_params = list(coefficient = 1:2),
                         rda_params = NULL, method = "ls",
                         big = FALSE) {
  ### Añadir filtro sondas con NAs
  
  
  if (verbose){
    message("Creating the model")
  }
  
  ## Create linear model
  if (is.null(model)){
    ## Get number of variables of interest
    model <- formula(paste("~", paste(variable_names, collapse = " + ")))
    num_vars <- ncol(createModel(set, model, warnings))

        if ("(Intercept)" %in% colnames(num_vars)){
      num_vars <- num_vars - 1
    }
    
    if (!is.null(covariable_names)){
      model <- formula(paste("~", paste(c(variable_names, covariable_names), 
                                            collapse = " + ")))
    } 
  } 
  model <- createModel(set, model, warnings)
  
  set <- set[, rownames(model)]
  
  
  
  ## Get matrix
  if (is(set, "ExpressionSet")){
    mat <- Biobase::exprs(set)
  } else if (is(set, "GenomicRatioSet")){
    mat <- minfi::getBeta(set)
    if (!betas) {
      mat[mat == 0] <- 1e-3
      mat[mat == 1] <- 1 - 1e-3
      mat <- minfi::logit2(mat)
    }
  } else if (is(set, "SummarizedExperiment")){
    mat <- SummarizedExperiment::assay(set)
  } else {
    stop("set must be an ExpressionSet, MethylationSet, GenomicRatioSet or SummarizedExperiment.")
  }
  
  ## Get Annotation
  fFun <- getFeatureDataFun(set)
  
  if (sva){
    if (verbose){
      message("Computing SVA")
    }
    model <- runSVA(mat, model, num_vars = num_vars,
                    big = big)
  }
  
  # Run Differential Mean analysis
  if (verbose){
    message("Probe Analysis started")
  }

  resList <- list()
  
  if ("DiffMean" %in% analyses){
    diffmean <- tryCatch(do.call(runDiffMeanAnalysis, 
                                 c(list(set = mat, model = model, weights = weights,
                                        resultSet = FALSE, 
                                        warnings = warnings, method = method), DiffMean_params)), 
                         error = function(e) e)
    resList$DiffMean <- list(result = diffmean, error = ifelse(is(diffmean, "try-error"), diffmean, NA))
  }
  if ("DiffVar" %in% analyses){
    
  diffvar <- tryCatch(do.call(runDiffVarAnalysis, 
                     c(list(set = mat, model = model, resultSet = FALSE, 
                            warnings = warnings), DiffVar_params)), 
                     error = function(e) e)
  resList$DiffVar <- list(result = diffvar, error = ifelse(is(diffvar, "try-error"), diffvar, NA))
  }
 
  if (!missing(range)){
    rda <- do.call(runRDA, c(list(set = set, model = model, num_vars = num_vars, 
                                  resultSet = FALSE, range = range), rda_params))
    resList$RDA <- list(result = rda, error = NA)
  } 
  
  res <- create_resultset(fOrigin = "runPipeline", 
                          lResults = resList,
                          fData = list(main = fFun(set)),
                          lOptions = list(sva = sva))
  
  return(res)
}

runSVA <- function (mat, model, num_vars, big=TRUE){
  df <- data.frame(model)
  model0 <- model[, -c(2:(1+num_vars)), drop = FALSE]
  
  if (big){
   Y.r <- t(resid(lm(t(mat) ~ ., df)))
   n.sv <- isva::EstDimRMT(Y.r, FALSE)$dim + 1
   if (n.sv > 0){
     svobj <- SmartSVA::smartsva.cpp(mat, model, model0, n.sv = n.sv)
     model <- cbind(model, svobj$sv)
   }
   else{
     IQRs <- apply(mat, 1, IQR)
     sv <- sva::sva(mat[IQRs > quantile(IQRs, prob=0.9), ],
                    mod=model, mod0=model0)
     cnames <- c(colnames(model), paste0("SV", 1:sv$n))
     model <- cbind(model, sv$sv)
     colnames(model) <- cnames
   }
  }
  return(model)
}

context("Pipeline")

library(minfiData)
# miniset <- MsetEx[1:100, ]
# pheno <- data.frame(pData(MsetEx))
# pheno$geno <- c("aa", "AA", "aA", "AA", "AA", "aA")
# set <- prepareMethylationSet(miniset, pheno)

set <- mapToGenome(ratioConvert(MsetEx[1:10, ]), mergeManifest = TRUE)
set <- set[apply(getBeta(set), 1, function(x) all(!is.na(x))), ]

eset <- ExpressionSet(matrix(runif(400, max = 15), 100))
annot <- data.frame(chr = rep("chr1", 100), start = 1:100, 
                    end = 1001:1100)
fData(eset) <- annot
pData(eset) <- data.frame(sex = rep(c("H", "M", "H", "M")))

range <- GRanges("chr1:1-2100")

test_that("run Pipeline", {
  results <- runPipeline(set = set, variable_names = "sex")
  expect_match(class(results), "ResultSet")
  results <- runPipeline(set = eset, variable_names = "sex", verbose = TRUE)
  expect_match(class(results), "ResultSet")

  # results <- runPipeline(set = eset, variable_names = "sex", sva = TRUE)
  expect_match(class(results), "ResultSet")
  
  results <- runPipeline(set = eset, variable_names = "sex", range = range)
  expect_match(class(results), "ResultSet")
  
})

#' Run differential variance analysis
#' 
#' Run differential variance analysis. This analysis can only be run with categorical 
#' variables. This function relies on \code{varFit} from missMethyl package.
#' 
#' @export runDiffVarAnalysis
#' 
#' @param set Matrix, \code{GenomicRatioSet}, \code{SummarizedExperiment} or 
#' \code{ExpressionSet}. 
#' @param model Model matrix or formula to get model matrix from \code{set}. 
#' @param coefficient Numeric with the coefficients used to make the groups. If NULL, all
#' possible groups will be computed. 
#' @param resultSet Should results be encapsulated in a \code{resultSet}? (Default: TRUE)
#' @param betas If \code{set} is a \code{GenomicRatioSet}, should beta values be
#' used? (Default: TRUE)
#' @param warnings Should warnings be displayed? (Default:TRUE) 
#' @param ... Further arguments passed to \code{varFit}.
#' @return \code{MArrayLM} or \code{resultSet} with the result of the differential
#' variance analysis.
#' @examples
#' if (require(minfiData)){
#'  mvalues <- getM(MsetEx)[1:100, ]
#'  model <- model.matrix(~ Sample_Group, data = pData(MsetEx)) 
#'  res <- runDiffVarAnalysis(mvalues, model)
#'  res
#' }
runDiffVarAnalysis <- function(set, model, coefficient = NULL,  
                            resultSet = TRUE, betas = TRUE, warnings = TRUE, ...) {
  
  ## Create model matrix from formula
  if (is(model, "formula")){
    model <- createModel(set, model)
    set <- set[, rownames(model)]
  }
  
  
  ## Get matrix
  if (is(set, "ExpressionSet")){
    mat <- Biobase::exprs(set)
  } else if (is(set, "GenomicRatioSet")){
    mat <- minfi::getBeta(set)
    if (!betas) {
      mat[mat == 0] <- 1e-3
      mat[mat == 1] <- 1 - 1e-3
      mat <- minfi::logit2(mat)
    }
  }else if (is(set, "SummarizedExperiment")){
    mat <- SummarizedExperiment::assay(set)
  } else if (is.matrix(set)){
    mat <- set
  } else {
    stop("set must be a matrix, ExpressionSet, MethylationSet, GenomicRatioSet or SummarizedExperiment.")
  }
  if (any(is.na(mat))){
    stop("runDiffVarAnalysis does not support NAs in the omic data.")
  }
  
  fFun <- getFeatureDataFun(set)
  
  fit <- missMethyl::varFit(mat, model, coef = coefficient, ...)
  
  if (resultSet){
    fit <- MultiDataSet::create_resultset(fOrigin = "DiffVarAnalysis", fData = list(main = fFun(set)), 
                                          lResults = list(DiffVar = list(result = fit)))
  } 
  fit
}

#' Plot values of a feature
#' 
#' Plot values of a feature splitted by one or two variables.
#' 
#' @export
#' @param set \code{ExpressionSet}, \code{GenomicRatioSet} or \code{SummarizedExperiment.}
#' @param feat Numeric with the index of the feature or character with its name.
#' @param variables Character vector with the names of the variables to be used 
#' in the splitting. Two variables is the maximum allowed.
#' @param betas If \code{set} is a \code{GenomicRatioSet}, should beta values be
#' used? (Default: TRUE)
#' @return A plot is generated on the current graphics device.
#' @examples
#' if (require(minfiData)){
#' set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#'  plotFeature(set, 1, variables = "Sample_Group")
#'  }

plotFeature <- function(set, feat, variables = colnames(pheno)[1], betas = TRUE){
  if (length(feat) != 1){
    stop("feat must contain only one value.")
  }
  methplot <- FALSE
  ## Get matrix
  if (is(set, "ExpressionSet")){
    vals <- Biobase::exprs(set)[feat, ]
  } else if (is(set, "GenomicRatioSet")){
    vals <- minfi::getBeta(set)[feat, ]
    methplot <- TRUE
    if (!betas) {
      vals[vals == 0] <- 1e-3
      vals[vals == 1] <- 1 - 1e-3
      vals <- minfi::logit2(vals)
      methplot <- FALSE
    }
  } else if (is(set, "SummarizedExperiment")){
    vals <- SummarizedExperiment::assay(set)[feat, ]
  } else {
    stop("set must be an ExpressionSet, GenomicRatioSet or SummarizedExperiment.")
  }
  
  ## Change getters depending on set class
  if (is(set, "eSet")){
    pheno <- Biobase::pData(set)
  }
  else if (is(set, "SummarizedExperiment")){
    pheno <- SummarizedExperiment::colData(set)
  } 
  
  if (length(variables) < 1 || length(variables) > 2){
    stop("variables must have one or two values.")
  }  
  if (!all(variables %in% colnames(pheno))){
    stop("Not all variables are present in set phenodata.")
  }
  
  values <- as.vector(vals)
  
  datamatrix <- data.frame(Values = values, factor = pheno[ , variables[1]])
  if (length(variables) == 2){
    datamatrix <- cbind(datamatrix, factor2 = pheno[ , variables[2]] )
  }
  if (is.numeric(datamatrix$factor)){
    p <- ggplot2::ggplot(datamatrix, ggplot2::aes_string(x = "factor", y = "Values")) + 
      ggplot2::geom_point() +
      ggplot2::geom_smooth(method = lm, se = FALSE) + 
      ggplot2::scale_x_continuous(name = "") + 
      ggplot2::ggtitle(feat)
    if (length(variables) == 2){
      p <- p +  ggplot2::facet_grid(. ~ factor2) + 
        ggplot2::scale_x_continuous(name = variables[2])
    }
  }else{  
    groups <- split(datamatrix, datamatrix$factor)
    means <- data.frame(means = sapply(groups, function(x) mean(x[ , 1])), 
                        factor = names(groups),
                        Values = 0)
    p <- ggplot2::ggplot(datamatrix, 
                         ggplot2::aes_string(x = "factor", y = "Values", fill = "factor")) + 
      ggplot2::geom_violin(alpha = 1/2) +
      ggplot2::geom_point(alpha = 1/2, position = ggplot2::position_jitter(width = 0.1)) + 
      ggplot2::geom_errorbar(data = means, 
                             ggplot2::aes_string(x = "factor", ymax = "means", 
                                                 ymin = "means"),
                             width = 0.4, size = 1) +
      ggplot2::scale_x_discrete(name = "") + ggplot2::ggtitle(feat) + 
      ggplot2::scale_fill_discrete(name = variables[1])
    if (length(variables) == 2){
      p <- p +  ggplot2::facet_grid(. ~ factor2) + 
        ggplot2::scale_x_discrete(name = variables[2])
    }
  }
  if (methplot){
    p <- p + ggplot2::scale_y_continuous("Methylation (Beta)", limits = c(0, 1))
  }
    
 p
}

# Return the function to get feature data
#
# Returns the function required to get feature data depending on the incoming 
# object. This function ensures the standarization of \code{resultSet} fData.
# @param set \code{eSet}, \code{SummarizedExperiment} or \code{RangedSummarizedExperiment}
getFeatureDataFun <- function(set){
  if (is(set, "eSet")){
    fFun <- function(set) {
      df <- Biobase::fData(set)
      ## Change position by start to harmonize fDatas
      if ("position" %in% colnames(df)){
        colnames(df)[colnames(df) == "position"] <- "start"
        df$end <- df$start
      }
      if ("chr" %in% colnames(df)){
        colnames(df)[colnames(df) == "chr"] <- "chromosome"
      } 
      df
    }
  } else if (is(set, "RangedSummarizedExperiment")){
    fFun <- function(set) { 
      df <- as.data.frame(SummarizedExperiment::rowRanges(set), 
                          stringsAsFactors = FALSE)
      colnames(df)[1] <- "chromosome"
      df[, 1] <- as.character(df[, 1])
      df
    } 
  } else if (is(set, "SummarizedExperiment")){
    fFun <- SummarizedExperiment::rowData
  } else if (is.matrix(set)){
    fFun <- function(set) data.frame(matrix(vector(), nrow(set), 0))
  }
  return(fFun)
}

#' Obtain probe results from a \code{ResultSet}
#' 
#' It computes the statistics from the \code{MArrayLM} computed with 
#' \code{DiffMeanAnalysis} or \code{DiffVarAnalysis}. This function allows to 
#' specify the contrasts and to get F-statistics for a group of variables. 
#' 
#' @name getProbeResults
#' @aliases getProbeResults 
#' 
#' @param object ResultSet
#' @param rid Name of the results: "DiffMean" for mean differences, "DiffVar" for
#' variance differences. (Default: DiffMean)
#' @param coef Number of the coefficient used to compute the statistics. If a vector
#' is supplied, F-statistics evaluating the global effect of the coefficients are
#' computed. (Default: 2).
#' @param contrast Matrix of contrasts 
#' @param fNames Names of the columns of \code{object} fData that will be added to 
#' the results data.frame.
#' @param ... Further arguments passed to \code{getAssociation}.
#' @return data.frame with the probe results. 
#' @export
getProbeResults <- function(object, rid = "DiffMean", coef = 2, contrast = NULL, 
                        fNames = c("chromosome", "start"), robust = FALSE, ...) {
    
  stopifnot(is(object, "ResultSet"))
  
    if (is.numeric(rid)){
      rid <- names(object)[rid]
    }
    
    if (rid == "DiffMean") {
      res <- getAssociation(object, rid = rid, coef = coef, contrast = contrast, 
                            fNames = fNames, robust = robust, ...)
    } else if (rid == "DiffVar"){
      res <- object@results[[rid]]$result
      
      if (!is.null(contrast)){
        res <- missMethyl::contrasts.varFit(res, contrast)
      }
      res <- missMethyl::topVar(res, coef = coef, number = nrow(res))
      
      ## Add fData to results
      if (!is.null(fNames)){
        fData <- object@fData[[1]]
        
        if (!all(fNames %in% colnames(fData))){
          stop("All fNames must be present in ResultSet fData.")
        }
        
        res <- cbind(res, fData[rownames(res), fNames])  
      }
    } else {
      stop("This ResultSet does not contain probe results")
    }
    
    return(res)
  }

#' Defunct functions 
#' 
#' These functions are defunct and no longer available.
#' 
#' @name MEAL-defunct
#' 
#' @details Defunct functions are: multiCorrMethExprs, DAPipeline, DAProbe, DARegion, 
#' RDAset, filterSet, plotBestFeatures, preparePhenotype, createRanges, prepareMethylationSet,
#' calculateRelevantSNPs, correlationMethSNPs, explainedVariance, normalSNP, plotLM
#' 
#' Defunct classes are: analysisRegionResults, analysisResults
NULL

#' Exports results data.frames to csv files.
#' 
#' Exports results to csv files. If more than one variable is present, subfolders
#' with the name of the variable are created. For each variable, four files will 
#' be generated: probeResults.csv, dmrCateResults.csv, bumphunterResults.csv
#' and blockFinderResults.csv
#' 
#' @name exportResults
#' @aliases exportResults 
#' 
#' @export 
#' 
#' @param object \code{ResultSet}
#' @param dir Character with the path to export.
#' @param prefix Character with a prefix to be added to all file names.
#' @param fNames Names of the columns of \code{object} fData that will be added to 
#' the results data.frame.
#' @return Files are saved into the given folder.
#' @examples
#' if (require(minfiData)){
#' set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#' methyOneVar <- runPipeline(set, variable_names = "sex")
#' exportResults(methyOneVar)
#' }
exportResults <- function(object, dir = "./", prefix = NULL, 
                          fNames = c("chromosome", "start")) {
  stopifnot(is(object, "ResultSet"))
  if (substr(dir, nchar(dir), nchar(dir)) != "/"){
    dir <- paste0(dir, "/")
  }
  if (!file.exists(dir)){
    dir.create(dir)
  }
  if ("DiffMean" %in% names(object)) {
    temp <- getProbeResults(object, rid = "DiffMean", fNames = fNames)
    write.csv2(temp, file = paste0(dir, prefix, "DiffMeanResults.csv"))
  }
  if ("DiffVar" %in% names(object)) {
    temp <- getProbeResults(object, rid = "DiffVar", fNames = fNames)
    write.csv2(temp, file = paste0(dir, prefix, "DiffVarResults.csv"))
  }
  if ("dmrcate" %in% names(object)) {
    temp <- getAssociation(object, rid = "dmrcate")
    write.csv2(temp, file = paste0(dir, prefix, "dmrCateResults.csv"))
  }
  if ("bumphunter" %in% names(object)) {
    temp <- getAssociation(object, rid = "bumphunter")
    write.csv2(temp, file = paste0(dir, prefix, "bumphunterResults.csv"))
  }
  if ("blockFinder" %in% names(object)) {
    temp <- getAssociation(object, rid = "blockFinder")
    write.csv2(temp, file = paste0(dir, prefix, "blockFinderResults.csv")) 
  }
}

# context("Export files")
# 
# library(minfiData)
# set <- prepareMethylationSet(getBeta(MsetEx)[1:10,], pheno = data.frame(pData(MsetEx)))
# methyOneVar <- DAPipeline(set, variable_names = "sex", probe_method = "ls")
# methyTwoVar <- DAPipeline(set, variable_names =  "person", probe_method = "ls")
# dir.create("./testfolder")
# setwd("./testfolder")
# 
# test_that("One variable, export same folder", {
#   exportResults(methyOneVar)
#   csvs <- dir( pattern = ".csv")
#   expect_equal(length(csvs), 4)
#   expect_equal(csvs, c("blockFinderResults.csv","bumphunterResults.csv", "dmrCateResults.csv",
#                        "probeResults.csv"))  
# })
# 
# test_that("One variable, export different folder", {
#   exportResults(methyOneVar, dir = "./test2")
#   csvs <- dir(path = "./test2",  pattern = ".csv")
#   expect_equal(length(csvs), 4)
#   expect_equal(csvs, c("blockFinderResults.csv","bumphunterResults.csv", "dmrCateResults.csv",
#                        "probeResults.csv"))  
# })
# 
# test_that("Two variables, export different folder", {
#   exportResults(methyTwoVar, dir = "./test3")
#   dirs <- dir(path = "./test3")
#   expect_equal(length(dirs), 2)
#   expect_equal(dirs, c("personid2", "personid3"))  
# })
# 
# 
# test_that("Two variables, choose one, export different folder", {
#   exportResults(methyTwoVar, dir = "./test4", vars = "personid2")
#   csvs <- dir(path = "./test4",  pattern = ".csv")
#   expect_equal(length(csvs), 4)
#   expect_equal(csvs, c("blockFinderResults.csv","bumphunterResults.csv", "dmrCateResults.csv",
#                        "probeResults.csv"))  
# })
# 
# test_that("Wrong variables", {
#   expect_error(exportResults(methyTwoVar, vars = character()), "Vars introduced are not present in the results object.")
#   expect_error(exportResults(methyTwoVar, vars = "cot"), "Vars introduced are not present in the results object.")
# })
# 
# test_that("One variable, add prefix export different folder", {
#   exportResults(methyOneVar, dir = "./test5", prefix = "p")
#   csvs <- dir(path = "./test5",  pattern = ".csv")
#   expect_equal(length(csvs), 4)
#   expect_equal(csvs, c("pblockFinderResults.csv","pbumphunterResults.csv", "pdmrCateResults.csv",
#                        "pprobeResults.csv"))  
# })
# 
# setwd("../")
# unlink("./testfolder", recursive = TRUE)

#' Run blockFinder
#' 
#' @details This function has been deprecated and will be defunct in the new version.
#' 
#' @export
#' 
#' @param set \code{GenomicRatioSet}, \code{eSet} derived object or 
#' \code{SummarizedExperiment}
#' @param model Model matrix or formula to get model matrix from \code{set}. 
#' @param coefficient Numeric with the column of model matrix used in the analysis.
#' (Default: 2)
#' @param blockfinder_cutoff Numeric with the minimum cutoff to include a probe
#' in a block. (Default: 0.1)
#' @param num_permutations Numeric with the number of permutations run to compute 
#' the blocks p-value. (Default: 0)
#' @param verbose Logical value. Should the function be verbose? (Default: FALSE)
#' @param resultSet Should results be encapsulated in a \code{resultSet}? (Default: TRUE)
#' @param ... Further arguments passed to \code{blockFinder}.
#' @return data.frame or \code{resultSet} with the result of \code{blockFinder}
#' @seealso \code{\link[minfi]{blockFinder}}

runBlockFinder <- function(set, model, coefficient = 2, blockfinder_cutoff = 0.1, 
                           num_permutations = 0, resultSet = FALSE, 
                           verbose = FALSE, ...) {
  
  .Deprecated()
}

#' Calculate RDA for a set
#' 
#' Perform RDA calculation for a \code{AnalysisRegionResults}. Feature values will
#' be considered the matrix X and phenotypes the matrix Y. Adjusting for covariates 
#' is done using a model matrix passed in covarsmodel.
#'  
#' @export runRDA
#' @param set \code{MethylationSet}, \code{ExpressionSet} or \code{matrix}
#' @param model Model matrix or formula to get model matrix from \code{set}. 
#' @param num_vars Numeric with the number of variables in the matrix for which the
#' analysis will be performed. Compulsory if equation is not null. 
#' @param range \code{GenomicRanges} with the region used for RDA
#' @param betas If \code{set} is a \code{GenomicRatioSet}, should beta values be
#' used? (Default: TRUE)
#' @param resultSet Should results be encapsulated in a \code{resultSet}? (Default: TRUE)
#' @param num_permutations Numeric with the number of permutations run to compute 
#' the p-value. (Default: 1e4)
#' @param ... Further arguments passed to \code{rda}.
#' @return Object of class \code{rda} or \code{resultSet} 
#' @seealso \code{\link[vegan]{rda}}
#' @examples
#' if (require(minfiData)){
#' set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#' model <- model.matrix(~set$age)
#' rda <- runRDA(set, model)
#' rda
#' }
runRDA <- function(set, model, num_vars = ncol(model), range, betas = FALSE, 
                   resultSet = TRUE, num_permutations = 1e4, ...){
  
  ## Create model matrix from formula
  if (is(model, "formula")){
    model <- createModel(set, model)
    set <- set[, rownames(model)]
  }
  
  ## Get original matrix to compute region significance
  if (is(set, "ExpressionSet")){
    orimat <- Biobase::exprs(set)
  } else if (is(set, "GenomicRatioSet")){
    orimat <- minfi::getBeta(set)
    if (!betas) {
      orimat[orimat == 0] <- 1e-3
      orimat[orimat == 1] <- 1 - 1e-3
      orimat <- minfi::logit2(orimat)
    }
  } else if (is(set, "SummarizedExperiment")){
    orimat <- SummarizedExperiment::assay(set)
  } else if (is.matrix(set)){
    orimat <- set
  } else {
    stop("set must be a matrix, ExpressionSet, MethylationSet, GenomicRatioSet or SummarizedExperiment.")
  }
  
  
  if(!missing(range)){
    stopifnot(is(range, "GenomicRanges"))
    if (is(set, "eSet")){
      grset <- GenomicRanges::makeGRangesFromDataFrame(fData(set), 
                            start.field = c("position", "start"), 
                            end.field = c("position", "end"))
      names(grset) <- rownames(set)
      grset <- IRanges::subsetByOverlaps(grset, range)
      set <- set[names(grset), ]
    } else if (is(set, "RangedSummarizedExperiment")){
      set <- IRanges::subsetByOverlaps(set, range)
    } else if (is(set, "SummarizedExperiment")){
      grset <- GenomicRanges::makeGRangesFromDataFrame(rowData(set), 
                                                       start.field = c("position", "start"), 
                                                       end.field = c("position", "end"))
      names(grset) <- rownames(set)
      grset <- IRanges::subsetByOverlaps(grset, range)
      set <- set[names(grset), ]
    } else{
      stop("range parameter only works with eSet or SummarizedExperiment derived objects.")
    }
    
  } 
  
  ## Get region matrix
  if (is(set, "ExpressionSet")){
    mat <- Biobase::exprs(set)
  } else if (is(set, "GenomicRatioSet")){
    mat <- minfi::getBeta(set)
    if (!betas) {
      mat[mat == 0] <- 1e-3
      mat[mat == 1] <- 1 - 1e-3
      mat <- minfi::logit2(mat)
    }
  } else if (is(set, "SummarizedExperiment")){
    mat <- SummarizedExperiment::assay(set)
  } else if (is.matrix(set)){
    mat <- set
  } else {
    stop("set must be a matrix, ExpressionSet, MethylationSet, GenomicRatioSet or SummarizedExperiment.")
  }
  
  varsmodel <- model[, seq_len(num_vars), drop = FALSE]
  if(num_vars == ncol(model)) {
    covarsmodel <- NULL
  }else {
    covarsmodel <- model[, (num_vars+1):ncol(model), drop = FALSE]
  }
  res <- vegan::rda(t(mat), varsmodel, covarsmodel, ...)
  res$pval <- vegan::anova.cca(res, permutations = permute::how(nperm = num_permutations))[["Pr(>F)"]][1]
  res$rdaR2 <- vegan::RsquareAdj(res)$r.squared
  
  regvals <- computeRDAR2(fullMat = orimat, varsmodel = varsmodel, 
                          covarsmodel = covarsmodel, featNum = nrow(mat), 
                          R2 = res$rdaR2, num_permutations = num_permutations)
  
  res$globalR2 <- regvals["globalR2"] 
  res$globalpval<- regvals["globalpval"]

  if (resultSet)
  {
    res <- create_resultset("runRDA", lResults = 
                              list(RDA = list(result = res, error = NA)),  
                            fData = list(), lOptions = list())
  }
  res
  return(res)
}

context("RDA calculation")

library(minfiData)
set <- mapToGenome(ratioConvert(MsetEx[1:10, ]), mergeManifest = TRUE)

range <- GenomicRanges::GRanges(seqnames=Rle("chrY"), 
                                ranges = IRanges(3000000, end=12300000))
test_that("runRDA", {
  rda <- runRDA(set = set, ~ sex)
  expect_true(is(rda, "ResultSet"))
  # rda2 <- runRDA(set = set, ~ sex + status, num_vars = 2)
  # expect_equal(class(rda), c("rda", "cca"))
  # rda <- runRDA(set = set, ~ sex, range = range)
  # expect_equal(class(rda), c("rda", "cca"))
})

# Create model.matrix from formula and omic set
# 
# Given an \code{eSet} or \code{SummarizedExperiment}, create a model matrix using
# formula. If there are missing values in the phenotype, the original set is subsetted
# to complete cases.
# 
#  
# @param set \code{eSet} or \code{SummarizedExperiment}
# @param model Formula with the model
# @param warnings Should warnings be displayed? (Default:TRUE)
# @return Matrix with the model matrix. 
createModel <- function(set, model, warnings = TRUE){
  ## Change getters depending on set class
  if (is(set, "eSet")){
    pFun <- Biobase::pData
  }
  else if (is(set, "SummarizedExperiment")){
    pFun <- SummarizedExperiment::colData
  } else{
    stop("set must be an eSet or a SummarizedExperiment derived object")
  }
  
  ## Create model matrix
  model <- model.matrix(model, pFun(set))
  
  ## Check missing data
  if (nrow(model) != ncol(set)){
    if (warnings){
      warning("There are some missing values in the samples data. Only complete cases will be used.")
    }
    set <- set[, rownames(model)]      
  }
  
  ## Check other paramaters
  if (ncol(set) == 0 | nrow(set) == 0){
    stop("The set is empty.")
  }
  if (ncol(model) == 0 | nrow(model) == 0){
    stop("The model matrix is empty.")
  }
  if (ncol(set) != nrow(model)){
    stop("The number of samples is different in the set and in the model")
  }
  
  return(model)
}

# context("Create Model")
# 
# test_that("Check colnames", {
#   pheno <- data.frame(a = c("a", "b", "c", "b", "a"), d = 1:5, e = runif(5))
#   model <- createModel(pheno)
#   expect_equal(ncol(model), 5)
#   names <- letters[1:5]
#   model <- createModel(pheno, names = names)
#   expect_equal(colnames(model), names)
# })
# 
# 
# test_that("Check equation", {
#   pheno <- data.frame(a = c("a", "b", "a", "b", "a"), d = 1:5, e = runif(5))
#   model <- createModel(pheno, equation = "~ a")
#   expect_equal(ncol(model), 4)
#   model <- createModel(pheno, equation = "~ a + d")
#   expect_equal(ncol(model), 4)
#   model <- createModel(pheno, equation = "~ a + d + a:d")
#   expect_equal(ncol(model), 5)
#   expect_equal(colnames(model), c("(Intercept)", "ab", "d", "e", "ab:d"))
#   names <- letters[1:5]
#   model <- createModel(pheno, equation = "~ a + d + a:d", names = names)
#   expect_equal(colnames(model), names)
# })
# 
# test_that("Check wrong variables", {
#   pheno <- data.frame()
#   expect_error(createModel(pheno), "data is empty.")
#   pheno <- 1:5
#   expect_error(createModel(pheno), "data is not a valid data.frame")
#   pheno <- data.frame(a = c("a", "b", "a", "b", "a"), d = 1:5, e = runif(5))
#   equation <- character()
#   expect_error(createModel(pheno, equation), "Equation is an invalid formula.")
#   equation <- "a"
#   expect_error(createModel(pheno, equation), "Equation is an invalid formula.")
#   equation <- "~ A"
#   expect_error(createModel(pheno, equation), "A model can't be created with this equation.")
#   expect_warning(createModel(pheno, names = character()), 
#                  "names variable is empty.")
#   expect_warning(createModel(pheno, equation = "~ a + d", names = character()), 
#                  "names variable is empty.")
#   expect_warning(createModel(pheno, names = "a"), 
#                  "Length of names is different than column names of model. Names won't be changed.")
#   expect_warning(createModel(pheno, equation = "~ a + d", names = "a"), 
#                  "Length of names is different than column names of model. Names won't be changed.")
# })
# 
# 

#' Run bumphunter
#' 
#' @details This function has been deprecated and will be defunct in the new version.
#' 
#' @export
#' 
#' @param set \code{GenomicRatioSet}, \code{eSet} derived object or 
#' \code{SummarizedExperiment}
#' @param model Model matrix or formula to get model matrix from \code{set}. 
#' @param coefficient Numeric with the column of model matrix used in the analysis.
#' (Default: 2)
#' @param bumphunter_cutoff Numeric with the minimum cutoff to include a probe
#' in a block. (Default: 0.1)
#' @param num_permutations Numeric with the number of permutations run to compute 
#' the bumps p-value. (Default: 0)
#' @param betas If \code{set} is a \code{GenomicRatioSet}, should beta values be
#' used? (Default: TRUE)
#' @param check_perms Logical. Should we check that there are less bumps than 
#' \code{bumps_max}? This parameter only applies when \code{num_permutations} is 
#' greater than 0. (Default: TRUE)
#' @param bumps_max Numeric with the maximum number of bumps used in the permutation.
#' This parameter only applies when \code{num_permutations} is greater than 0. 
#' (Default: 30000)
#' @param verbose Logical value. Should the function be verbose? (Default: FALSE)
#' @param resultSet Should results be encapsulated in a \code{resultSet}? (Default: TRUE)
#' @param ... Further arguments passed to \code{bumphunter}.
#' @return data.frame or \code{resultSet} with the result of \code{bumphunter}
#' @seealso \code{\link[minfi]{bumphunter}}
runBumphunter <- function(set, model, coefficient = 2, bumphunter_cutoff = 0.1,
                          num_permutations = 0, bumps_max = 30000, betas = TRUE, 
                          check_perms = FALSE, verbose = FALSE, 
                          resultSet = FALSE, ...){

  .Deprecated()
}

#' Computes the correlation between methylation and expression
#' 
#' Estimates the correlation between methylation and expression. When there are known 
#' variables that affect methylation and/or expression, their effect can be substracted
#' using a linear model and then the residuals are used. 
#' 
#' For each cpg, a range is defined by the position of the cpg plus the flank parameter
#' (upstream and downstream). Only those expression probes that are entirely in 
#' this range will be selected. For these reason, it is required that the \code{ExpressionSet} 
#' contains a featureData with the chromosome and the starting and ending positions 
#' of the probes.  
#' 
#' @export correlationMethExprs
#' 
#' @param multiset \code{MultiDataSet} containing a \code{methylation} and an 
#' \code{expression} slots.
#' @param meth_set_name Character vector with the name of the \code{MultiDataSet}'s slot containing methylation
#' data.
#' @param exprs_set_name Character vector with the name of the \code{MultiDataSet}'s slot containing expression
#' data.
#' @param vars_meth Character vector with the names of the variables that will be
#' used to obtain the methylation residuals. By default, none is used and residuals 
#' are not computed.
#' @param vars_exprs Character vector with the names of the variables that will
#' be used to obtain the expression residuals. By default, none is used and
#' residuals are not computed.
#' @param sel_cpgs Character vector with the name of the CpGs used in the analysis. If empty, all the CpGs of the 
#' methylation set will be used. 
#' @param flank Numeric with the number of pair bases used to define the cpg-expression 
#' probe pairs.
#' @param betas If \code{set} is a \code{GenomicRatioSet}, should beta values be
#' used? (Default: TRUE)
#' @param num_cores Numeric with the number of cores to be used.
#' @param verbose Logical value. If TRUE, it writes out some messages indicating progress. 
#' If FALSE nothing should be printed.
#' @return Data.frame with the results of the linear regression:
#' \itemize{
#'  \item cpg: Name of the cpg
#'  \item exprs: Name of the expression probe
#'  \item beta: coefficient of the methylation change
#'  \item se: standard error of the beta 
#'  \item P.Value: p-value of the beta coefficient
#'  \item adj.P.Val: q-value computed using B&H
#' }
correlationMethExprs <- function(multiset, 
                                 meth_set_name = NULL, exprs_set_name = NULL,
                                 vars_meth = NULL, vars_exprs = NULL, 
                                 sel_cpgs, flank = 250000, betas = TRUE,
                                 num_cores = 1, verbose = TRUE){
  
  # To DO
  ## Añadir check a vars_meth y vars_exprs
  
  ######################################################################################
  ## Data Checking
  # Check object is a MultiDataSet
  if (!is(multiset, "MultiDataSet")){
    stop("multiset must be a MultiDataSet")
  }
  
  # Check meth_set_name and exprs_set_name are characters
  if (!(is.character(meth_set_name) | is.null(meth_set_name))){
    stop("meth_set_name must be a character.")
  } 
  if (!(is.character(exprs_set_name) | is.null(exprs_set_name))){
    stop("exprs_set_name must be a character.")
  } 
  
  # Add the dataset type to the names
  meth_set_name <- paste(c("methylation", meth_set_name), collapse = "+")
  exprs_set_name <- paste(c("expression", exprs_set_name), collapse = "+")
  
  
  # Check our object has the right sets
  if (!all(c(meth_set_name, exprs_set_name) %in% names(multiset))){
    stop("multiset must contain meth_set_name and exprs_set_name.")
  }
  
  if (!all(c(meth_set_name, exprs_set_name) %in% rowRangesElements(multiset))){
    stop("multiset must contain meth_set_name and exprs_set_name.")
  }
  
  # Check that flank is numeric, positive and is only one number
  if (!is(flank, "numeric") || length(flank) > 1 || flank < 0){
    stop("flank must be a positive integer")
  }
  #####################################################################################
  
  #####################################################################################
  ## Preparation of data
  #############################
  ## Preparation of data 1
  # Select only our sets
  multiset <- multiset[, c(meth_set_name, exprs_set_name)]
  
  ## Select common samples
  multiset <- commonSamples(multiset)
  #############################
  ## Preparation of data 2
  
  mset <- multiset[[meth_set_name]]
  eset <- multiset[[exprs_set_name]]
  
  #############################
  ## Preparation of data 3
  if (!missing(sel_cpgs)){
    if (!is.character(sel_cpgs) | length(sel_cpgs) == 0){
      stop("sel_cpgs must be a character vector with the name of the CpGs to be used.")
    } else{
      mset <- mset[sel_cpgs, ]
    }
  }
  
  #############################
  ## Preparation of data 4
  # Compute Methylation-Expression pairs
  rangesMeth <- rowRanges(multiset)[[meth_set_name]]
  rangesMeth <- rangesMeth[featureNames(mset)]
  rangesExprs <- rowRanges(multiset)[[exprs_set_name]]
  #####################################################################################
  
  #####################################################################################
  ## Implementation of the algorithm
  #############################
  ## Implementation of the algorithm 1
  start(rangesMeth) <- start(rangesMeth) - flank
  end(rangesMeth) <- end(rangesMeth) + flank
  pairs <- GenomicRanges::findOverlaps(rangesExprs, rangesMeth,  type = "within")
  pairs <- data.frame(cpg = rownames(mset)[S4Vectors::subjectHits(pairs)], 
                      exprs = rownames(eset)[S4Vectors::queryHits(pairs)], 
                      stringsAsFactors = FALSE)
  if (nrow(pairs) == 0){
    warning("There are no expression probes in the range of the cpgs. An empty data.frame will be returned.")
    return(data.frame(cpg = character(0), exprs = character(0), Beta = integer(0), 
                      se = integer(0), P.Value = integer(0), adj.P.val = integer(0)))
  }
  #############################
  ## Filter sets to only features in the pairs
  mset <- mset[unique(pairs[ , 1]), ]
  eset <- eset[unique(pairs[ , 2]), ]
  
  
  if (verbose){
    message("Computing residuals")
  }
  
  #############################
  ## Implementation of the algorithm 2
  # Computing residuals
  methres <- setResidues(mset, variable_names = vars_meth, betas = betas)
  exprsres <- setResidues(eset, variable_names = vars_exprs)
  #############################
  if (verbose){
    message("Computing correlation Methylation-Expression")
  }
  
  #############################
  ## Implementation of the algorithm 3
  residualsCorr <- function (methy_res, exprs_res){
    fit <- lm(exprs_res ~ methy_res)
    return(c(summary(fit)$coef[2, 1], summary(fit)$coef[2,2], summary(fit)$coef[2,4]))
  }
  regvals <- mclapply(1:nrow(pairs), 
                      function(x) residualsCorr(methres[pairs[x, 1], ], exprsres[pairs[x, 2], ]), 
                      mc.cores = num_cores)
  #####################################################################################
  
  #####################################################################################
  ## Formatting the results
  res <- data.frame(pairs, t(data.frame(regvals)))
  colnames(res) <- c("cpg", "exprs", "Beta", "se", "P.Value")
  res$adj.P.Val <- p.adjust(res$P.Value, "BH")
  res <- res[order(res$adj.P.Val), ]
  rownames(res) <- NULL
  res
  #####################################################################################
}

setResidues <- function(set, variable_names, betas = TRUE){
  
  if (is(set, "ExpressionSet")){
    res <- Biobase::exprs(set)
  } else if (is(set, "GenomicRatioSet")){
    res <- minfi::getBeta(set)
    if (!betas) {
      res[res == 0] <- 1e-3
      res[res == 1] <- 1 - 1e-3
      res <- minfi::logit2(res)
    }
  } else if (is(set, "SummarizedExperiment")){
    res <- SummarizedExperiment::assay(set)
  } 
  
  if (!is.null(variable_names)){
    model <- formula(paste("~", paste(variable_names, collapse = " + ")))
    ## Get number of variables of interest
    model <- createModel(set, model)
    
    res <- residuals(limma::lmFit(res, model), res)
  }
 
  return(res)
}

context("Correlation")

library(minfiData)
set <- mapToGenome(ratioConvert(MsetEx[1:10,  1:4]), mergeManifest = TRUE)
colData(set)$geno <-  c("aa", "AA", "aA", "AA")

eset <- ExpressionSet(matrix(runif(12, max = 15), 3))
fData(eset) <- data.frame(chromosome = rep("chrY", 3), start = c(10000, 6700000, 5000000), 
                          end = c(100090, 6710000, 90000000))
featureNames(eset) <- letters[1:3]
rownames(fData(eset)) <- featureNames(eset)
pData(eset) <- data.frame(sex = c("H", "M", "H", "M"))
sampleNames(eset) <- sampleNames(set)

multi <- createMultiDataSet()
multi <- add_genexp(multi, eset)
multi <- add_methy(multi, set)

test_that("Check lineal correlation is working", {
  results <- correlationMethExprs(multiset = multi)
  expect_match(class(results), "data.frame")
  expect_equal(nrow(results), 1)  
  results <- correlationMethExprs(multiset = multi, vars_meth = "person")
  expect_equal(nrow(results), 1)  
  results <- correlationMethExprs(multiset = multi, vars_exprs = "sex")
  expect_equal(nrow(results), 1)  
  results <- correlationMethExprs(multiset = multi, vars_meth = "person",
                                  vars_exprs = "sex")
  expect_equal(nrow(results), 1)  
  results <- correlationMethExprs(multiset = multi, flank = 5000000)
  fData(eset)[] <- data.frame(chromosome = rep("chrY", 3), start = c(10000, 6700000, 5000000), 
                            end = c(100090, 7710000, 90000000))
  multi2 <- add_genexp(multi, eset, overwrite = TRUE)
  expect_warning(results <- correlationMethExprs(multiset = multi2),  
                 "There are no expression probes in the range of the cpgs. An empty data.frame will be returned.")
  expect_equal(nrow(results), 0)
})

test_that("Empty and wrong variables linear", {
  expect_error(correlationMethExprs(multiset = eset), "multiset must be a MultiDataSet")
  emptymulti <- createMultiDataSet()
  expect_error(correlationMethExprs(multiset = emptymulti), "multiset must contain meth_set_name and exprs_set_name.")

 
  multi <- add_genexp(multi, eset, overwrite = TRUE)
  expect_error(correlationMethExprs(multiset = multi, flank = -5000000), "flank must be a positive integer")
  expect_error(correlationMethExprs(multiset = multi, flank = "233"), "flank must be a positive integer")
  expect_error(correlationMethExprs(multiset = multi, flank = c(50000, 100)), "flank must be a positive integer")
  # expect_warning(correlationMethExprs(multiset = multi, vars_meth = "cot"),  
  #                "vars_meth is/are not a valid column of the mset phenoData. No residues will be computed")  
  # expect_warning(correlationMethExprs(multiset = multi, vars_exprs = "cot"),  
  #                "vars_exprs is/are not a valid column of the eset phenoData. No residues will be computed")  
})

#' MEAL (Methylation and Expression AnaLizer): Package for analysing methylation 
#' and expression data
#' 
#' MEAL is a package designed to facilitate the analysis methylation and expression 
#' data. The package can analyze one dataset and can find correlations between
#' methylation and expression data. MEAL has a vignette that explains the main
#' functionalities of the package. 
#' 
#' @docType package
#' @name MEAL
#' 
#' @import BiocGenerics
#' @import Biobase
#' @import methods
#' @import MultiDataSet
#' @importFrom GenomicRanges seqnames start end makeGRangesFromDataFrame findOverlaps GRanges
#' @importFrom ggplot2 aes aes_string alpha facet_grid geom_bar geom_errorbar geom_hline geom_line geom_point geom_polygon geom_smooth geom_text geom_vline ggplot ggtitle position_jitter scale_colour_manual scale_fill_manual scale_x_continuous scale_y_continuous theme
#' @importFrom graphics legend points text
#' @importFrom IRanges IRanges
#' @importFrom limma lmFit contrasts.fit eBayes topTable 
#' @importFrom matrixStats rowVars
#' @importFrom minfi logit2 ilogit2 bumphunter GenomicRatioSet cpgCollapse blockFinder getBeta
#' @importFrom parallel mclapply
#' @importFrom permute how
#' @importFrom S4Vectors Rle runValue subjectHits queryHits
#' @importFrom stats contrasts cor.test formula lm model.matrix p.adjust prcomp qbeta qt
#' @importFrom SummarizedExperiment assay
#' @importFrom utils write.csv2
#' @importFrom vegan anova.cca ordilabel RsquareAdj rda
NULL

#' Get all probes related to a gene
#' 
#' Given a \code{ResultSet} and a gene name returns the results of the 
#' analysis of all the probes of the gene. 
#' 
#' @name getGeneVals
#' @aliases getGeneVals
#' @export
#' @param object \code{ResultSet}
#' @param gene Character with the name of the gene
#' @param rid Name of the results: "DiffMean" for mean differences, "DiffVar" for
#' variance differences. (Default: DiffMean)
#' @param genecol Character with the column of \code{object} fData with the gene
#' information
#' @param fNames Names of the columns of \code{object} fData that will be added to 
#' the results data.frame.
#' @param ... Further arguments passed to \code{getProbeResults}
#' @return data.frame with the results of the analysis of the probes belonging to 
#' the gene 
#' @examples
#' \dontrun{
#' if (require(minfiData)){
#'  set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#' methyOneVar <- runPipeline(set, variable_names = "sex")
#' getGeneVals(methyOneVar, "TSPY4")
#' }
#' }
getGeneVals <- function(object, gene, rid = 1, genecol = "genes", 
                        fNames = c("chromosome", "start"), ...){
  
  stopifnot(is(object, "ResultSet"), is(gene, "character"))
  
  res <- getProbeResults(object, rid, fNames = c(fNames, genecol), ...)
  
  stopifnot(genecol %in% colnames(res))
  
  genelist <- strsplit(res[[genecol]], ";")
  mask <- vapply(genelist, function(y) gene %in% y, logical(length(gene)))
  if (length(gene) > 1){
    mask <- as.logical(colSums(mask))
  }
  return(res[mask, , drop = FALSE])
  if (nrow(res) == 0){
    warning("Gene name is not present in the results. An empty data.frame will be returned.")
  }
  res
}

# context("Get genes")
# 
# library(minfiData)
# set <- prepareMethylationSet(getBeta(MsetEx)[1:10,], pheno = data.frame(pData(MsetEx)))
# methyOneVar <- DAPipeline(set, variable_names = "sex", probe_method = "ls")
# methyTwoVar <- DAPipeline(set, variable_names =  "person", probe_method = "ls")
# 
# test_that("One category", {
#   res <- getGeneVals(methyOneVar, "TSPY4") 
#   expect_equal(length(res), 1)
#   expect_equal(nrow(res[[1]]), 2)  
#   expect_equal(nrow(getGeneVals(methyOneVar, "TTTY14")[[1]]), 1)  
# })
# 
# test_that("Two categories", {
#   res <- getGeneVals(methyTwoVar, "TSPY4") 
#   expect_equal(length(res), 2)
#   expect_equal(nrow(res[[1]]), 2)  
#   expect_equal(nrow(getGeneVals(methyTwoVar, "TTTY14")[[1]]), 1)  
# })
# 
# test_that("Wrong variables", {
#  expect_warning(getGeneVals(methyTwoVar, "TS12"), "Gene name is not present in the results. A list with empty data.frames will be returned.")
#   expect_error(getGeneVals(methyTwoVar, 12), "gene must be a character vector")
#   })

#' Get the top features associated with the RDA
#' 
#' Get a list of the features significantly associated to the first two RDA components
#' 
#' @name topRDAhits
#' @export
#' 
#' @param object \code{ResultSet} 
#' @param tPV numeric with the p-value threshold. Only features with a p-values below this threshold will be shown. 
#' @return data.frame with the features, the component, the correlation and the p-value
#' @examples
#' if (require(minfiData) & require(GenomicRanges)){
#' set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
#' model <- model.matrix(~set$sex)
#' rda <- runRDA(set, model)
#' topRDAhits(rda)
#' }
topRDAhits <- function(object, tPV = 0.05){
    
    stopifnot("RDA" %in% names(object))
    rda <- getAssociation(object, rid = "RDA")
    rdavals <- rda$CCA$u
    
    if (ncol(rdavals) > 2){
      rdavals <- rdavals[, 1:2]
    }
    vals <- vegan::ordiYbar(rda, "partial")
    resmat <- vapply(1:ncol(rdavals), function(x) vapply(1:ncol(vals), function(y){
      res <- cor.test(rdavals[,x], vals[,y])
      res <- c(res$estimate, res$p.value)
      names(res) <- c("cor", "pval")
      res
    }, numeric(2)), numeric(ncol(vals)*2))
    colnames(resmat) <- colnames(rdavals)
    cors <- resmat[seq(1, nrow(resmat), 2), , drop = FALSE]
    pvals <- resmat[seq(2, nrow(resmat), 2), , drop = FALSE]
    rownames(cors) <- colnames(vals)
    rownames(pvals) <- colnames(vals)
    adjpvals <- vapply(1:ncol(pvals), function(x) p.adjust(pvals[,x]), numeric(nrow(pvals)))
    signif.feats <- which(adjpvals < tPV, arr.ind = TRUE)
    res <- data.frame(feat = rownames(cors)[signif.feats[ , 1]], 
                      RDA = colnames(cors)[signif.feats[ , 2]],
                      cor = cors[signif.feats], 
                      P.Value = pvals[signif.feats], 
                      adj.P.Val = adjpvals[signif.feats])
    res <- res[order(res$P.Value), ]
    res
  }

---
title: "Expression and Methylation Analysis with MEAL"
author: 
  - name: Carlos Ruiz
    affiliation: 
    - ISGlobal, Centre for Research in Environmental Epidemiology (CREAL), Barcelona, Spain
    - Bioinformatics Research Group in Epidemiology 
  - name: Juan R. González
    affiliation: 
    - ISGlobal, Centre for Research in Environmental Epidemiology (CREAL), Barcelona, Spain
    - Bioinformatics Research Group in Epidemiology  
    email: juanr.gonzalez@isglobal.org 
package: MEAL
output: 
  BiocStyle::html_document:
    number_sections: true
    toc_float: yes
bibliography: ./vignette.bib
vignette: >
  %\VignetteIndexEntry{Expression and Methylation Analysis with MEAL}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---


```{r setup, include=FALSE}
knitr::opts_chunk$set(message = FALSE, warning = FALSE)
```


# Introduction

Experiments comprising different omic data analyses are becoming common. In this cases, the researchers are interested in analyzing the different omic types independently as well as finding association between the different layers.

In this vignette, we present how address to an analysis of methylation and expression data using `r Biocpkg("MEAL")`. We will show how to analyze each data type independently and how to integrate the results of both analyses. We will also exemplify how to run an analysis when we are interested in our target region.

We will use the data from `r Rpackage("brgedata")` package. This package contains data from a spanish children cohort. We will work with the methylation and the expression data encapsulated in a `GenomicRatioSet` and in an `ExpressionSet` respectively. In our analysis, we will evaluate the effect of sex on methylation and expression. We will also deeply explore changes in a region of chromosome 11 around MMP3, a gene differently expressed in men and women (chr11:102600000-103300000).

Let's start by loading the required packages and data:
```{r Load_Packages, message = FALSE}
library(MEAL)
library(brgedata)
library(MultiDataSet)
library(missMethyl)
library(minfi)
library(GenomicRanges)
library(ggplot2)
```

From the six objects of `r Rpackage("brgedata")`, we will focus on those containing methylation and expression data: brge_methy and brge_gexp.

brge_methy is a `GenomicRatioSet` with methylation data corresponding to the Illumina Human Methylation 450K. It contains 392277 probes and 20 samples. The object contains 9 phenotypic variables (age, sex and cell types proportions):

```{r Methylation_Data}
data(brge_methy)
brge_methy
colData(brge_methy)
```

brge_gexp is an `ExpressionSet` with the expression data corresponding to an Affimetrix GeneChip Human Gene 2.0 ST Array. It contains 67528 features and 100 samples as well as two phenotypic variables (age and sex):

```{r Expression_Data}
data(brge_gexp)
brge_gexp
lapply(pData(brge_gexp), table)
```

Finally, we will create a `GenomicRanges` with our target region:

```{r}
targetRange <- GRanges("chr11:102600000-103300000")
```

Let's illustrate the use of this package by analyzing the effect of the sex in methylation and expression. First, a genome wide analysis will be performed and then the region funcionalities will be introduced. It should be noticed that methylation and expression analyses will include hypothesis testing and visualitzation. 

# Methylation Analysis

## Running the analyses

This demonstration will show how to perform a probe methylation analysis. To run a more complete analysis, including DMR methods, see `r Biocpkg("MEAL")` vignette. 

The function `runDiffMeanAnalysis` detects probes differentially methylated using `limma`. This function accepts a `GenomicRatioSet`, an `ExpressionSet` or a `SummarizedExperiment`, so it can analyze methylation and gene expression data. We can pass our model to function `runDiffMeanAnalysis` with the parameter `model`. We are interested in evaluating the effect of sex and We will include cell counts as covariables to adjust their effect:

```{r Meth_Analysis}
cellCounts <- colnames(colData(brge_methy))[3:9]
methRes <- runDiffMeanAnalysis(set = brge_methy, 
                       model = formula(paste("~ sex +", paste(cellCounts, collapse = "+"))))
methRes
names(methRes)
```

The analysis generates a `ResultSet` object containing the results of our analysis. The first step after running the pipeline will be evaluate how this model fits the data. A common way to evaluate the goodness of fit of these models is by making a QQplot on the p-values of the linear regression. This plot compares the distribution of p-values obtained in the analysis with a theoretical distribution. Our plot also shows the lambda, a measure of inflation. If lambda is higher than 1, the p-values are smaller than expected. If it is lower than 1, p-values are bigger than expected. In both causes, the most common cause is that we should include more variables in the model.

We can get a QQplot with the function `plot`. When applied to a `ResultSet`, we can make different plots depending on the method. We can select the name of the result with the parameter `rid` and the kind of plot with the parameter `type`. We will set `rid` to "DiffMean" to use limma results and `type` to "qq" to get a QQplot:

```{r Plot QQ 1}
plot(methRes, rid = "DiffMean", type = "qq")
```

When there are few CpGs modicated, we expect points in the QQplot lying on the theoretical line. However, as all the CpGs from the sexual chromosomes will be differentially methylated between males and females, we obtained a S-shape. In this case, lambda is lower than 1 (0.851). 

There are different ways to address this issue. An option is to include other covariates in our model. When we do not have more covariates, as it is our case, we can apply Surrogate Variable Analysis (SVA) to the data. SVA is a statistical technique that tries to determine hidden covariates or SV (Surrogate Variables) based on the measurements. This method is very useful to correct inflation when we do not have the variables that are missing in the model. Our function `runPipeline` includes the parameter `sva` that runs SVA and includes these variables as covariates in the linear model. More information can be found in `r Biocpkg("MEAL")` vignette. 

We can get an overview of the results using a Manhattan plot. A Manhattan plot represents the p-value of each CpG versus their location in the genome. This representation is useful to find genomic regions exhibiting differentiated behaviours. We can get a Manhattan plot from a `ResultSet` using the function `plot` and setting `type` to "manhattan". We will highlight the CpGs of our target region by passing a `GenomicRanges` to highlight argument. `GenomicRanges` passed to `plot` should have chromosomes coded as number:

```{r Manhattans}
targetRangeNum <- GRanges("11:102600000-103300000")
plot(methRes, rid = "DiffMean", type = "manhattan", main = "Differences in Means", highlight = targetRangeNum)
```

There are no genomic regions with a lot of CpGs with differences in means. Our target region does not seem to be very different from the rest. On the other hand, chromosome X has a lot of CpGs with big differences in variance. Our target region does not look different. 

# Expression Analysis

## Running the analysis

The expression analysis can be performed in the same way than methylation. The main difference is that an `ExpressionSet` will be used instead of a `GenomicRatioSet`. We will run the same model than for methylation. We will check the effect of sex on gene expression and we will further evaluate our target region:

```{r Exp show}
targetRange <- GRanges("chr11:102600000-103300000")
gexpRes <- runDiffMeanAnalysis(set = brge_gexp, model = ~ sex)
names(gexpRes)
```

As we did with methylation data, the first step will be check the goodness of our model. To do so, we will run a QQ-plot using the p-values of the differences of means:

```{r Plot QQ exp 1}
plot(gexpRes, rid = "DiffMean", type = "qq")
```

Most of the p-values lie on the theoretical line and the lambda is close to 1 (0.94). Therefore, we will not need to include other covariates in our model. 

We will now get a general overview of the results. In gene expression data, it is very common to use a Volcano plot which shows the change in expression of a probe against their p-value. We can easily make these plots using the function `plot` and setting the argument `type` to __volcano__:

```{r Volcano gexp}
plot(gexpRes, rid = "DiffMean", type = "volcano") + ggtitle("Differences in Means")
```

Several transcripts have a big difference in mean gene expression and a very small p-value. Transcripts with a positive fold change have a higher expression in boys than in girls and are placed in chromosome Y (name starts by TC0Y). Transcripts with a negative fold change, have a lower expression in boys than in girls and are placed in chromosome X (name starts by TC0X). 

We can get an overview of the genome distribution of these transcripts by plotting a Manhattan plot. We will also highlight the transcripts of our target region:

```{r Manhattans gexp}
targetRange <- GRanges("chr11:102600000-103300000")
plot(gexpRes, rid = "DiffMean", type = "manhattan", main = "Differences in Means", highlight = targetRangeNum)
```

There are several transcripts with highly significant differences in means in chromosomes X and Y. However, transcripts in our target region are not significantly changed.

# Methylation and gene expression integration

We have included in `r Biocpkg("MEAL")` two ways to assess association between gene expression and DNA methylation. The first option is to simultaneously examine the gene expression and DNA methylation results of a target region. We can do this with the function `plotRegion` and passing two ResultSets.

We will examine the methylation and gene expression results in our target region. For the sake of clarity, we will only print DiffMean and bumphunter results.

```{r Regional plot 2 exp}
plotRegion(rset = methRes, rset2 = gexpRes, range = targetRange)
```

Differences in methylation are represented in black and differences in gene expression are represented in blue. We can see that CpGs with high differences are near transcripts with highest differences expression.

We can get a quantitative estimate of the association between methylation and gene expression using the function `correlationMethExprs`. This function applies a method previously described in [@Steenaard2015]. CpGs and expression probes are paired by proximity. Expression probes are paired to a CpG if they are completely inside a range of $\pm$ 250Kb from the CpG. This distance of 250Kb is set by default but it can be changed with the parameter flank (whose units are bases). The correlation between methylation and expression is done by a linear regression. 

To account for technical (e.g. batch) or biological (e.g. sex, age) artifacts, a model including those variables (z) is fitted: $$ x_{ij} = \sum_{k = 1}^K{\beta_{ik} z_{kj}} + r_{ij}, i = 1, ..., P $$ where $x_{ij}$ is the methylation or expression level of probe i (P is the total number of probes) for individual j, $\beta_{ik}$ is the effect of variable k at probe i, $z_{kj}$ is the value of variable k (K is the total number of covariates) for indivudal j and $r_{ij}$ is the residual value of probe i and individual j. The residuals of methylation and expression are used to assess the correlation.

Therefore, let's create a new `MultiDataSet` with expression and methylation data. The function `createMultiDataSet` creates an empty `MultiDataSet`. Then, we can add gene expression and methylation datasets with the function `add_genexp` and `add_methy`:

```{r New Multi Meth Exp}
multi <- createMultiDataSet()
multi <- add_genexp(multi, brge_gexp)
multi <- add_methy(multi, brge_methy)
```

In this analysis, we will only take into account the probes in our target region. We can subset our MultiDataSet passing a `GenomicRanges` to the third position of `[`: 

```{r}
multi.filt <- multi[, , targetRange]
```

Now, we will compute the correlation between the CpGs and the expression probes. 

```{r Corr Meth Exp}
methExprs <- correlationMethExprs(multi.filt)
head(methExprs)
```

The results shows the CpG name, the expression probe, the linear regression coeffcient and its standard error, and the p-value and adjusted p-value (using B&H). Results are ordered by the adjusted p-value, so we can see than in our data there are no correlated CpGs-expression probes.

# Conclusions

This case example shows the main functionalities of `r Biocpkg("MEAL")` package. It has been shown how to evaluate the effect of a phenotype on the methylation and on the expression at genome wide and at region level. Finally, the integration between methylation and expression is tested by checking if there are expression probes correlated to the methylated probes.

```{r SessionInfo}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ResultsSet_topRDAhits.R
\name{topRDAhits}
\alias{topRDAhits}
\title{Get the top features associated with the RDA}
\usage{
topRDAhits(object, tPV = 0.05)
}
\arguments{
\item{object}{\code{ResultSet}}

\item{tPV}{numeric with the p-value threshold. Only features with a p-values below this threshold will be shown.}
}
\value{
data.frame with the features, the component, the correlation and the p-value
}
\description{
Get a list of the features significantly associated to the first two RDA components
}
\examples{
if (require(minfiData) & require(GenomicRanges)){
set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
model <- model.matrix(~set$sex)
rda <- runRDA(set, model)
topRDAhits(rda)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runDMRcate.R
\name{runDMRcate}
\alias{runDMRcate}
\title{Run DMRcate}
\usage{
runDMRcate(set, model, coefficient = 2, resultSet = FALSE, ...)
}
\arguments{
\item{set}{\code{GenomicRatioSet}, \code{eSet} derived object or 
\code{SummarizedExperiment}}

\item{model}{Model matrix or formula to get model matrix from \code{set}.}

\item{coefficient}{Numeric with the column of model matrix used in the analysis.
(Default: 2)}

\item{resultSet}{Should results be encapsulated in a \code{resultSet}? (Default: TRUE)}

\item{...}{Further arguments passed to \code{cpg.annotate} or \code{dmrcate}.}
}
\value{
data.frame or \code{resultSet} with the result of \code{bumphunter}
}
\description{
Run DMRcate
}
\details{
This function has been deprecated and will be defunct in the new version.
}
\seealso{
\code{\link[DMRcate]{dmrcate}}, \code{\link[DMRcate]{cpg.annotate}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ResultSet_getGeneVals.R
\name{getGeneVals}
\alias{getGeneVals}
\title{Get all probes related to a gene}
\usage{
getGeneVals(
  object,
  gene,
  rid = 1,
  genecol = "genes",
  fNames = c("chromosome", "start"),
  ...
)
}
\arguments{
\item{object}{\code{ResultSet}}

\item{gene}{Character with the name of the gene}

\item{rid}{Name of the results: "DiffMean" for mean differences, "DiffVar" for
variance differences. (Default: DiffMean)}

\item{genecol}{Character with the column of \code{object} fData with the gene
information}

\item{fNames}{Names of the columns of \code{object} fData that will be added to 
the results data.frame.}

\item{...}{Further arguments passed to \code{getProbeResults}}
}
\value{
data.frame with the results of the analysis of the probes belonging to 
the gene
}
\description{
Given a \code{ResultSet} and a gene name returns the results of the 
analysis of all the probes of the gene.
}
\examples{
\dontrun{
if (require(minfiData)){
 set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
methyOneVar <- runPipeline(set, variable_names = "sex")
getGeneVals(methyOneVar, "TSPY4")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Defunct.R
\name{MEAL-defunct}
\alias{MEAL-defunct}
\title{Defunct functions}
\description{
These functions are defunct and no longer available.
}
\details{
Defunct functions are: multiCorrMethExprs, DAPipeline, DAProbe, DARegion, 
RDAset, filterSet, plotBestFeatures, preparePhenotype, createRanges, prepareMethylationSet,
calculateRelevantSNPs, correlationMethSNPs, explainedVariance, normalSNP, plotLM

Defunct classes are: analysisRegionResults, analysisResults
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ResultSet_plotRDA.R
\name{plotRDA}
\alias{plotRDA}
\title{Plot RDA results}
\usage{
plotRDA(object, pheno = data.frame(), n_feat = 5, main = "RDA plot", alpha = 1)
}
\arguments{
\item{object}{\code{ResultSet}}

\item{pheno}{data.frame with the variables used to color the samples.}

\item{n_feat}{Numeric with the number of cpgs to be highlighted. Default: 5.}

\item{main}{Character with the plot title.}

\item{alpha}{Numeric with the alpha level for colour transparance. Default: 1; no
transparency.}
}
\value{
A plot is generated on the current graphics device.
}
\description{
Plot RDA results
}
\examples{
if (require(minfiData)){
set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
model <- model.matrix(~set$sex)
rda <- runRDA(set, model)
plotRDA(rda, pheno = data.frame(factor(set$sex)))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getProbeResults.R
\name{getProbeResults}
\alias{getProbeResults}
\title{Obtain probe results from a \code{ResultSet}}
\usage{
getProbeResults(
  object,
  rid = "DiffMean",
  coef = 2,
  contrast = NULL,
  fNames = c("chromosome", "start"),
  robust = FALSE,
  ...
)
}
\arguments{
\item{object}{ResultSet}

\item{rid}{Name of the results: "DiffMean" for mean differences, "DiffVar" for
variance differences. (Default: DiffMean)}

\item{coef}{Number of the coefficient used to compute the statistics. If a vector
is supplied, F-statistics evaluating the global effect of the coefficients are
computed. (Default: 2).}

\item{contrast}{Matrix of contrasts}

\item{fNames}{Names of the columns of \code{object} fData that will be added to 
the results data.frame.}

\item{...}{Further arguments passed to \code{getAssociation}.}
}
\value{
data.frame with the probe results.
}
\description{
It computes the statistics from the \code{MArrayLM} computed with 
\code{DiffMeanAnalysis} or \code{DiffVarAnalysis}. This function allows to 
specify the contrasts and to get F-statistics for a group of variables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runRDA.R
\name{runRDA}
\alias{runRDA}
\title{Calculate RDA for a set}
\usage{
runRDA(
  set,
  model,
  num_vars = ncol(model),
  range,
  betas = FALSE,
  resultSet = TRUE,
  num_permutations = 10000,
  ...
)
}
\arguments{
\item{set}{\code{MethylationSet}, \code{ExpressionSet} or \code{matrix}}

\item{model}{Model matrix or formula to get model matrix from \code{set}.}

\item{num_vars}{Numeric with the number of variables in the matrix for which the
analysis will be performed. Compulsory if equation is not null.}

\item{range}{\code{GenomicRanges} with the region used for RDA}

\item{betas}{If \code{set} is a \code{GenomicRatioSet}, should beta values be
used? (Default: TRUE)}

\item{resultSet}{Should results be encapsulated in a \code{resultSet}? (Default: TRUE)}

\item{num_permutations}{Numeric with the number of permutations run to compute 
the p-value. (Default: 1e4)}

\item{...}{Further arguments passed to \code{rda}.}
}
\value{
Object of class \code{rda} or \code{resultSet}
}
\description{
Perform RDA calculation for a \code{AnalysisRegionResults}. Feature values will
be considered the matrix X and phenotypes the matrix Y. Adjusting for covariates 
is done using a model matrix passed in covarsmodel.
}
\examples{
if (require(minfiData)){
set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
model <- model.matrix(~set$age)
rda <- runRDA(set, model)
rda
}
}
\seealso{
\code{\link[vegan]{rda}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotRegion.R
\name{plotRegion}
\alias{plotRegion}
\title{Plot results in a genomic region}
\usage{
plotRegion(
  rset,
  range,
  results = names(rset),
  genome = "hg19",
  rset2,
  tPV = 5,
  fNames = c("chromosome", "start", "end"),
  fNames2 = c("chromosome", "start", "end")
)
}
\arguments{
\item{rset}{\code{ResultSet}}

\item{range}{\code{GenomicRanges} with the region coordinates}

\item{results}{Character with the analyses that will be included in the plot.
By default, all analyses available are included.}

\item{genome}{String with the genome used to retrieve transcripts annotation:
hg19, hg38, mm10. (Default: "hg19")}

\item{rset2}{Additional \code{ResultSet}}

\item{tPV}{Threshold for P-Value}

\item{fNames}{Names from rset fData}

\item{fNames2}{Names from rset2 fData}
}
\value{
Regional plot
}
\description{
Plot the results from the different analyses of a \code{ResultSet} in a specific
genomic region. It can plot all the results from \code{runPipeline}.
}
\details{
This plot allows to have a quick summary of the methylation or gene
expression analyses in a given region. If we use a \code{ResultSet} obtained 
from methylation data, transcripts annotation is obtained from archive. If we
use a \code{ResultSet} obtained from gene expression data, transcripts annotation
is taken from fData. 

This plot can be used to plot the results of one dataset (methylation or gene
expression) or to represent the association between methylation and gene 
expression data. If only one dataset is used, the p-values and the coefficients 
of DiffMean and DiffVar analyses are plotted. If we pass two \code{ResultSet}s, 
\code{rset} should contain methylation results and a \code{rset2} the gene expression 
results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlationMethExprs.R
\name{correlationMethExprs}
\alias{correlationMethExprs}
\title{Computes the correlation between methylation and expression}
\usage{
correlationMethExprs(
  multiset,
  meth_set_name = NULL,
  exprs_set_name = NULL,
  vars_meth = NULL,
  vars_exprs = NULL,
  sel_cpgs,
  flank = 250000,
  betas = TRUE,
  num_cores = 1,
  verbose = TRUE
)
}
\arguments{
\item{multiset}{\code{MultiDataSet} containing a \code{methylation} and an 
\code{expression} slots.}

\item{meth_set_name}{Character vector with the name of the \code{MultiDataSet}'s slot containing methylation
data.}

\item{exprs_set_name}{Character vector with the name of the \code{MultiDataSet}'s slot containing expression
data.}

\item{vars_meth}{Character vector with the names of the variables that will be
used to obtain the methylation residuals. By default, none is used and residuals 
are not computed.}

\item{vars_exprs}{Character vector with the names of the variables that will
be used to obtain the expression residuals. By default, none is used and
residuals are not computed.}

\item{sel_cpgs}{Character vector with the name of the CpGs used in the analysis. If empty, all the CpGs of the 
methylation set will be used.}

\item{flank}{Numeric with the number of pair bases used to define the cpg-expression 
probe pairs.}

\item{betas}{If \code{set} is a \code{GenomicRatioSet}, should beta values be
used? (Default: TRUE)}

\item{num_cores}{Numeric with the number of cores to be used.}

\item{verbose}{Logical value. If TRUE, it writes out some messages indicating progress. 
If FALSE nothing should be printed.}
}
\value{
Data.frame with the results of the linear regression:
\itemize{
 \item cpg: Name of the cpg
 \item exprs: Name of the expression probe
 \item beta: coefficient of the methylation change
 \item se: standard error of the beta 
 \item P.Value: p-value of the beta coefficient
 \item adj.P.Val: q-value computed using B&H
}
}
\description{
Estimates the correlation between methylation and expression. When there are known 
variables that affect methylation and/or expression, their effect can be substracted
using a linear model and then the residuals are used.
}
\details{
For each cpg, a range is defined by the position of the cpg plus the flank parameter
(upstream and downstream). Only those expression probes that are entirely in 
this range will be selected. For these reason, it is required that the \code{ExpressionSet} 
contains a featureData with the chromosome and the starting and ending positions 
of the probes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runBumphunter.R
\name{runBumphunter}
\alias{runBumphunter}
\title{Run bumphunter}
\usage{
runBumphunter(
  set,
  model,
  coefficient = 2,
  bumphunter_cutoff = 0.1,
  num_permutations = 0,
  bumps_max = 30000,
  betas = TRUE,
  check_perms = FALSE,
  verbose = FALSE,
  resultSet = FALSE,
  ...
)
}
\arguments{
\item{set}{\code{GenomicRatioSet}, \code{eSet} derived object or 
\code{SummarizedExperiment}}

\item{model}{Model matrix or formula to get model matrix from \code{set}.}

\item{coefficient}{Numeric with the column of model matrix used in the analysis.
(Default: 2)}

\item{bumphunter_cutoff}{Numeric with the minimum cutoff to include a probe
in a block. (Default: 0.1)}

\item{num_permutations}{Numeric with the number of permutations run to compute 
the bumps p-value. (Default: 0)}

\item{bumps_max}{Numeric with the maximum number of bumps used in the permutation.
This parameter only applies when \code{num_permutations} is greater than 0. 
(Default: 30000)}

\item{betas}{If \code{set} is a \code{GenomicRatioSet}, should beta values be
used? (Default: TRUE)}

\item{check_perms}{Logical. Should we check that there are less bumps than 
\code{bumps_max}? This parameter only applies when \code{num_permutations} is 
greater than 0. (Default: TRUE)}

\item{verbose}{Logical value. Should the function be verbose? (Default: FALSE)}

\item{resultSet}{Should results be encapsulated in a \code{resultSet}? (Default: TRUE)}

\item{...}{Further arguments passed to \code{bumphunter}.}
}
\value{
data.frame or \code{resultSet} with the result of \code{bumphunter}
}
\description{
Run bumphunter
}
\details{
This function has been deprecated and will be defunct in the new version.
}
\seealso{
\code{\link[minfi]{bumphunter}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runDiffMeanAnalysis.R
\name{runDiffMeanAnalysis}
\alias{runDiffMeanAnalysis}
\title{Run differential mean analysis}
\usage{
runDiffMeanAnalysis(
  set,
  model,
  weights = NULL,
  method = "ls",
  max_iterations = 100,
  betas = TRUE,
  resultSet = TRUE,
  warnings = TRUE
)
}
\arguments{
\item{set}{Matrix, \code{GenomicRatioSet}, \code{SummarizedExperiment} or 
\code{ExpressionSet}.}

\item{model}{Model matrix or formula to get model matrix from \code{set}.}

\item{weights}{weights used in the lmFit model.}

\item{method}{String indicating the method used in the regression: "ls" or 
"robust". (Default: "ls")}

\item{max_iterations}{Numeric indicating the maximum number of iterations
done in the robust method.}

\item{betas}{If \code{set} is a \code{GenomicRatioSet}, should beta values be
used? (Default: TRUE)}

\item{resultSet}{Should results be encapsulated in a \code{resultSet}? (Default: TRUE)}

\item{warnings}{Should warnings be displayed? (Default:TRUE)}
}
\value{
\code{MArrayLM} or \code{resultSet} with the result of the differential
mean analysis.
}
\description{
Run differential mean analysis using t-moderated statistics. This function relies
on \code{lmFit} from limma package.
}
\examples{
if (require(minfiData)){
 mvalues <- getM(MsetEx)[1:100, ]
 model <- model.matrix(~ Sample_Group, data = pData(MsetEx)) 
 res <- runDiffMeanAnalysis(mvalues, model, method = "ls")
 res
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MEAL.R
\docType{package}
\name{MEAL}
\alias{MEAL}
\title{MEAL (Methylation and Expression AnaLizer): Package for analysing methylation 
and expression data}
\description{
MEAL is a package designed to facilitate the analysis methylation and expression 
data. The package can analyze one dataset and can find correlations between
methylation and expression data. MEAL has a vignette that explains the main
functionalities of the package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runDiffVarAnalysis.R
\name{runDiffVarAnalysis}
\alias{runDiffVarAnalysis}
\title{Run differential variance analysis}
\usage{
runDiffVarAnalysis(
  set,
  model,
  coefficient = NULL,
  resultSet = TRUE,
  betas = TRUE,
  warnings = TRUE,
  ...
)
}
\arguments{
\item{set}{Matrix, \code{GenomicRatioSet}, \code{SummarizedExperiment} or 
\code{ExpressionSet}.}

\item{model}{Model matrix or formula to get model matrix from \code{set}.}

\item{coefficient}{Numeric with the coefficients used to make the groups. If NULL, all
possible groups will be computed.}

\item{resultSet}{Should results be encapsulated in a \code{resultSet}? (Default: TRUE)}

\item{betas}{If \code{set} is a \code{GenomicRatioSet}, should beta values be
used? (Default: TRUE)}

\item{warnings}{Should warnings be displayed? (Default:TRUE)}

\item{...}{Further arguments passed to \code{varFit}.}
}
\value{
\code{MArrayLM} or \code{resultSet} with the result of the differential
variance analysis.
}
\description{
Run differential variance analysis. This analysis can only be run with categorical 
variables. This function relies on \code{varFit} from missMethyl package.
}
\examples{
if (require(minfiData)){
 mvalues <- getM(MsetEx)[1:100, ]
 model <- model.matrix(~ Sample_Group, data = pData(MsetEx)) 
 res <- runDiffVarAnalysis(mvalues, model)
 res
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ResultSet_getRDAresults.R
\name{getRDAresults}
\alias{getRDAresults}
\title{Get a summary of RDA results}
\usage{
getRDAresults(object)
}
\arguments{
\item{object}{\code{ResultSet}}
}
\value{
Numeric vector with the RDA statistics
}
\description{
Get statistics from RDA result.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterResults.R
\name{filterResults}
\alias{filterResults}
\title{Filter the data.frame obtained from probe analysis}
\usage{
filterResults(results, range, position = "position", chr = "chromosome")
}
\arguments{
\item{results}{Data.frame with the results of probe analysis}

\item{range}{\code{GenomicRanges} with the desired range.}

\item{position}{Character with the name of the column containing the positions}

\item{chr}{Character with the name of the column containing the chromosome}
}
\value{
Data.frame with the results of the probes of the range
}
\description{
Filter the data.frame obtained from probe analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runBlockFinder.R
\name{runBlockFinder}
\alias{runBlockFinder}
\title{Run blockFinder}
\usage{
runBlockFinder(
  set,
  model,
  coefficient = 2,
  blockfinder_cutoff = 0.1,
  num_permutations = 0,
  resultSet = FALSE,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{set}{\code{GenomicRatioSet}, \code{eSet} derived object or 
\code{SummarizedExperiment}}

\item{model}{Model matrix or formula to get model matrix from \code{set}.}

\item{coefficient}{Numeric with the column of model matrix used in the analysis.
(Default: 2)}

\item{blockfinder_cutoff}{Numeric with the minimum cutoff to include a probe
in a block. (Default: 0.1)}

\item{num_permutations}{Numeric with the number of permutations run to compute 
the blocks p-value. (Default: 0)}

\item{resultSet}{Should results be encapsulated in a \code{resultSet}? (Default: TRUE)}

\item{verbose}{Logical value. Should the function be verbose? (Default: FALSE)}

\item{...}{Further arguments passed to \code{blockFinder}.}
}
\value{
data.frame or \code{resultSet} with the result of \code{blockFinder}
}
\description{
Run blockFinder
}
\details{
This function has been deprecated and will be defunct in the new version.
}
\seealso{
\code{\link[minfi]{blockFinder}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runPipeline.R
\name{runPipeline}
\alias{runPipeline}
\title{Perform differential methylation analysis}
\usage{
runPipeline(
  set,
  variable_names,
  covariable_names = NULL,
  model = NULL,
  weights = NULL,
  num_vars,
  sva = FALSE,
  betas = TRUE,
  range,
  analyses = c("DiffMean"),
  verbose = FALSE,
  warnings = TRUE,
  DiffMean_params = NULL,
  DiffVar_params = list(coefficient = 1:2),
  rda_params = NULL,
  method = "ls",
  big = FALSE
)
}
\arguments{
\item{set}{\code{GenomicRatioSet}, \code{eSet} derived object or 
\code{SummarizedExperiment}}

\item{variable_names}{Character vector with the names of the variables that
will be returned as result.}

\item{covariable_names}{Character vector with the names of the variables that
will be used to adjust the model.}

\item{model}{Model matrix or formula to get model matrix from \code{set}.}

\item{weights}{weights used in the lmFit model (default NULL)}

\item{num_vars}{Numeric with the number of variables in the matrix for which the
analysis will be performed. Compulsory if equation is not null.}

\item{sva}{Logical. Should Surrogate Variable Analysis be applied? 
(Default: FALSE)}

\item{betas}{If \code{set} is a \code{GenomicRatioSet}, should beta values be
used? (Default: TRUE)}

\item{range}{\code{GenomicRanges} with the region used for RDA}

\item{analyses}{Vector with the names of the analysis to be run (DiffMean and/or DiffVar).}

\item{verbose}{Logical value. If TRUE, it writes out some messages indicating progress. 
If FALSE nothing should be printed.}

\item{warnings}{Should warnings be displayed? (Default:TRUE)}

\item{DiffMean_params}{List with other parameter passed to \code{runBumphunter} 
function.}

\item{DiffVar_params}{List with other parameter passed to \code{runBumphunter} 
function.}

\item{rda_params}{List with other parameter passed to \code{runRDA} 
function.}

\item{method}{String indicating the method used in the regression: "ls" or 
"robust". (Default: "ls")}

\item{big}{Logical value indicating whether SmartSVA should be instead of SVA 
(TRUE recommended for methylation or when having large number of samples). 
Default is FALSE.}
}
\value{
\code{ResultSet} object
}
\description{
Wrapper for analysing differential methylation and expression at region and probe level.
}
\details{
This function is the main wrapper of the package. First, it simplifies the 
the set to only contain the common samples between phenotype and features. In addition,
it allows to change the class of the variables and to apply genomic models (more 
information on \code{preparePhenotype}). Afterwards, analysis per probe and per 
region are done merging the results in an \code{AnalysisResults} object.

Default linear model will contain a sum of the variables and covariables. If 
interactions are desired, a costum formula can be specified. In that case, variables
and covariables must also be specified in order to assure the proper work of the
resulting \code{AnalysisResult}. In addition, the number of variables of the model
for which the calculation will be done \strong{must} be specified.
}
\examples{
if (require(minfiData)){
set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
 res <- runPipeline(set, variable_names = "Sample_Group") 
 res
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runRegionAnalysis.R
\name{runRegionAnalysis}
\alias{runRegionAnalysis}
\title{Run different DMR detection methods}
\usage{
runRegionAnalysis(
  set,
  model,
  methods = c("blockFinder", "bumphunter", "DMRcate"),
  coefficient = 2,
  bumphunter_params = NULL,
  blockFinder_params = NULL,
  dmrcate_params = NULL,
  verbose = FALSE,
  resultSet = TRUE
)
}
\arguments{
\item{set}{\code{GenomicRatioSet}, \code{eSet} derived object or 
\code{SummarizedExperiment}}

\item{model}{Model matrix representing a linear model.}

\item{methods}{Character vector with the names of the methods used to estimate 
the regions. Valid names are: "blockFinder", "bumphunter" and "DMRcate".}

\item{coefficient}{Numeric with the index of the model matrix used to perform
the analysis.}

\item{bumphunter_params}{List with other parameter passed to \code{runBumphunter} 
function.}

\item{blockFinder_params}{List with other parameter passed to \code{runBlockFinder} 
function.}

\item{dmrcate_params}{List with other parameter passed to \code{runDMRcate} 
function.}

\item{verbose}{Logical value. Should the function be verbose? (Default: FALSE)}

\item{resultSet}{Should results be encapsulated in a \code{resultSet}? (Default: TRUE)}
}
\value{
List or \code{resultSet} with the result of the DMR detection methods.
}
\description{
Run different DMR detection methods
}
\details{
This function has been deprecated and will be defunct in the new version.
}
\examples{
if (require(minfiData)){
set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
 model <- model.matrix(~Sample_Group, data = pData(MsetEx))
 res <- runRegionAnalysis(set, model) 
 res
}
}
\seealso{
\code{\link[minfi]{bumphunter}}, \code{\link[minfi]{blockFinder}}, 
\code{\link[DMRcate]{dmrcate}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ResultSet_exportResults.R
\name{exportResults}
\alias{exportResults}
\title{Exports results data.frames to csv files.}
\usage{
exportResults(
  object,
  dir = "./",
  prefix = NULL,
  fNames = c("chromosome", "start")
)
}
\arguments{
\item{object}{\code{ResultSet}}

\item{dir}{Character with the path to export.}

\item{prefix}{Character with a prefix to be added to all file names.}

\item{fNames}{Names of the columns of \code{object} fData that will be added to 
the results data.frame.}
}
\value{
Files are saved into the given folder.
}
\description{
Exports results to csv files. If more than one variable is present, subfolders
with the name of the variable are created. For each variable, four files will 
be generated: probeResults.csv, dmrCateResults.csv, bumphunterResults.csv
and blockFinderResults.csv
}
\examples{
if (require(minfiData)){
set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
methyOneVar <- runPipeline(set, variable_names = "sex")
exportResults(methyOneVar)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFeature.R
\name{plotFeature}
\alias{plotFeature}
\title{Plot values of a feature}
\usage{
plotFeature(set, feat, variables = colnames(pheno)[1], betas = TRUE)
}
\arguments{
\item{set}{\code{ExpressionSet}, \code{GenomicRatioSet} or \code{SummarizedExperiment.}}

\item{feat}{Numeric with the index of the feature or character with its name.}

\item{variables}{Character vector with the names of the variables to be used 
in the splitting. Two variables is the maximum allowed.}

\item{betas}{If \code{set} is a \code{GenomicRatioSet}, should beta values be
used? (Default: TRUE)}
}
\value{
A plot is generated on the current graphics device.
}
\description{
Plot values of a feature splitted by one or two variables.
}
\examples{
if (require(minfiData)){
set <- ratioConvert(mapToGenome(MsetEx[1:10,]))
 plotFeature(set, 1, variables = "Sample_Group")
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeRDAR2.R
\name{computeRDAR2}
\alias{computeRDAR2}
\title{Compute signification of RDA test}
\usage{
computeRDAR2(
  fullMat,
  varsmodel,
  covarsmodel = NULL,
  featNum,
  R2,
  num_permutations = 1e+05 - 1
)
}
\arguments{
\item{fullMat}{Matrix with the whole genome expression or methylation values}

\item{varsmodel}{Matrix with the model}

\item{covarsmodel}{Matrix with the covariables model}

\item{featNum}{Numeric with the number of features of the RDA model}

\item{R2}{Numeric with the R2 of the RDA model}

\item{num_permutations}{Numeric with the number of permutations.}
}
\value{
Numeric vector with the probability of finding a region with the same number of probes
with a bigger R2 and the global R2.
}
\description{
Compare R2 obtained in our region of interest with the global R^2 and the R^2 of regions with 
the same number of probes.
}
